@@ -22,8 +22,6 @@
 #include <errno.h>
 
 #define COBJMACROS
-#define NONAMELESSUNION
-
 #include "windef.h"
 #include "winbase.h"
 #include "winuser.h"
@@ -46,22 +44,22 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(appwizcpl);
 
-#define GECKO_VERSION "2.47.3"
+#define GECKO_VERSION "2.47.4"
 #ifdef __i386__
 #define GECKO_ARCH "x86"
-#define GECKO_SHA "e5b9b06d3ce355646a8d2e72e044e37e1e0c8d18464eb1985adcd187a7f48e01"
+#define GECKO_SHA "26cecc47706b091908f7f814bddb074c61beb8063318e9efc5a7f789857793d6"
 #elif defined(__x86_64__)
 #define GECKO_ARCH "x86_64"
-#define GECKO_SHA "a53ee954392b6d1fe3d68545f6e4e2a97afbc8dc8b03a8b443349545ce139675"
+#define GECKO_SHA "e590b7d988a32d6aa4cf1d8aa3aa3d33766fdd4cf4c89c2dcc2095ecb28d066f"
 #else
 #define GECKO_ARCH ""
 #define GECKO_SHA "???"
 #endif
 
-#define MONO_VERSION "7.3.0"
+#define MONO_VERSION "8.1.0"
 #if defined(__i386__) || defined(__x86_64__)
 #define MONO_ARCH "x86"
-#define MONO_SHA "939e2f5669720d0d0fc7e305426622a11a33b7ae385c4d7e581e29ea264e22ff"
+#define MONO_SHA "0ed3ec533aef79b2f312155931cf7b1080009ac0c5b4c2bcfeb678ac948e0810"
 #else
 #define MONO_ARCH ""
 #define MONO_SHA "???"
@@ -110,8 +108,7 @@ static LPWSTR url = NULL;
 static IBinding *dwl_binding;
 static WCHAR *msi_file;
 
-extern const char * CDECL wine_get_version(void);
-
+static const char * (CDECL *p_wine_get_version)(void);
 static WCHAR * (CDECL *p_wine_get_dos_file_name)(const char*);
 
 static BOOL sha_check(const WCHAR *file_name)
@@ -119,8 +116,6 @@ static BOOL sha_check(const WCHAR *file_name)
     const unsigned char *file_map;
     HANDLE file, map;
     DWORD size, i;
-    BCRYPT_HASH_HANDLE hash = NULL;
-    BCRYPT_ALG_HANDLE alg = NULL;
     UCHAR sha[32];
     char buf[1024];
     BOOL ret = FALSE;
@@ -143,26 +138,16 @@ static BOOL sha_check(const WCHAR *file_name)
     if(!file_map)
         return FALSE;
 
-    if(BCryptOpenAlgorithmProvider(&alg, BCRYPT_SHA256_ALGORITHM, MS_PRIMITIVE_PROVIDER, 0))
-        goto end;
-    if(BCryptCreateHash(alg, &hash, NULL, 0, NULL, 0, 0))
-        goto end;
-    if(BCryptHashData(hash, (UCHAR *)file_map, size, 0))
-        goto end;
-    if(BCryptFinishHash(hash, sha, sizeof(sha), 0))
-        goto end;
+    if(!BCryptHash(BCRYPT_SHA256_ALG_HANDLE, NULL, 0, (UCHAR *)file_map, size, sha, sizeof(sha))) {
+        for(i=0; i < sizeof(sha); i++)
+            sprintf(buf + i * 2, "%02x", sha[i]);
 
-    for(i=0; i < sizeof(sha); i++)
-        sprintf(buf + i * 2, "%02x", sha[i]);
+        ret = !strcmp(buf, addon->sha);
+        if(!ret)
+            WARN("Got %s, expected %s\n", buf, addon->sha);
+    }
 
-    ret = !strcmp(buf, addon->sha);
-    if(!ret)
-        WARN("Got %s, expected %s\n", buf, addon->sha);
-
-end:
     UnmapViewOfFile(file_map);
-    if(hash) BCryptDestroyHash(hash);
-    if(alg) BCryptCloseAlgorithmProvider(alg, 0);
     return ret;
 }
 
@@ -205,7 +190,7 @@ static enum install_res install_from_dos_file(const WCHAR *dir, const WCHAR *sub
     HRESULT hr;
 
     size += lstrlenW( subdir ) + lstrlenW( file_name ) + 2;
-    if (!(path = heap_alloc( size * sizeof(WCHAR) ))) return INSTALL_FAILED;
+    if (!(path = malloc( size * sizeof(WCHAR) ))) return INSTALL_FAILED;
 
     lstrcpyW( path, dir );
     if (!wcsncmp( path, L"\\??\\", 4 ))  path[1] = '\\';  /* change \??\ into \\?\ */
@@ -219,10 +204,10 @@ static enum install_res install_from_dos_file(const WCHAR *dir, const WCHAR *sub
     if (FAILED( hr ))
     {
         ERR( "Failed to canonicalize %s, hr %#lx\n", debugstr_w(path), hr );
-        heap_free( path );
+        free( path );
         return INSTALL_NEXT;
     }
-    heap_free( path );
+    free( path );
 
     if (GetFileAttributesW( canonical_path ) == INVALID_FILE_ATTRIBUTES)
     {
@@ -245,7 +230,7 @@ static enum install_res install_from_unix_file(const char *dir, const WCHAR *sub
     if (p_wine_get_dos_file_name && (dos_dir = p_wine_get_dos_file_name( dir )))
     {
         ret = install_from_dos_file( dos_dir, subdir, file_name );
-        heap_free( dos_dir );
+        HeapFree( GetProcessHeap(), 0, dos_dir );
     }
     return ret;
 }
@@ -267,7 +252,7 @@ static HKEY open_config_key(void)
 
 static enum install_res install_from_registered_dir(void)
 {
-    char *package_dir;
+    char *package_dir, *new_package_dir;
     HKEY hkey;
     DWORD res, type, size = MAX_PATH;
     enum install_res ret;
@@ -276,24 +261,27 @@ static enum install_res install_from_registered_dir(void)
     if(!hkey)
         return INSTALL_NEXT;
 
-    package_dir = heap_alloc(size);
+    package_dir = malloc(size);
     res = RegGetValueA(hkey, NULL, addon->dir_config_key, RRF_RT_ANY, &type, (PBYTE)package_dir, &size);
     if(res == ERROR_MORE_DATA) {
-        package_dir = heap_realloc(package_dir, size);
-        res = RegGetValueA(hkey, NULL, addon->dir_config_key, RRF_RT_ANY, &type, (PBYTE)package_dir, &size);
+        new_package_dir = realloc(package_dir, size);
+        if(new_package_dir) {
+            package_dir = new_package_dir;
+            res = RegGetValueA(hkey, NULL, addon->dir_config_key, RRF_RT_ANY, &type, (PBYTE)package_dir, &size);
+        }
     }
     RegCloseKey(hkey);
     if(res == ERROR_FILE_NOT_FOUND) {
-        heap_free(package_dir);
+        free(package_dir);
         return INSTALL_NEXT;
     } else if(res != ERROR_SUCCESS || (type != REG_SZ && type != REG_EXPAND_SZ)) {
-        heap_free(package_dir);
+        free(package_dir);
         return INSTALL_FAILED;
     }
 
     ret = install_from_unix_file(package_dir, L"", addon->file_name);
 
-    heap_free(package_dir);
+    free(package_dir);
     return ret;
 }
 
@@ -305,7 +293,7 @@ static enum install_res install_from_default_dir(void)
 
     if ((package_dir = _wgetenv( L"WINEBUILDDIR" )))
     {
-        dir_buf = heap_alloc( lstrlenW(package_dir) * sizeof(WCHAR) + sizeof(L"\\..\\") );
+        dir_buf = malloc( wcslen(package_dir) * sizeof(WCHAR) + sizeof(L"\\..\\") );
         lstrcpyW( dir_buf, package_dir );
         lstrcatW( dir_buf, L"\\..\\" );
         package_dir = dir_buf;
@@ -315,7 +303,7 @@ static enum install_res install_from_default_dir(void)
     if (package_dir)
     {
         ret = install_from_dos_file(package_dir, addon->subdir_name, addon->file_name);
-        heap_free(dir_buf);
+        free(dir_buf);
     }
 
     if (ret == INSTALL_NEXT)
@@ -341,7 +329,7 @@ static WCHAR *get_cache_file_name(BOOL ensure_exists)
     }
     else if ((home_dir = _wgetenv( L"WINEHOMEDIR" )))
     {
-        if (!(cache_dir = heap_alloc( lstrlenW(home_dir) * sizeof(WCHAR) + sizeof(L"\\.cache") ))) return NULL;
+        if (!(cache_dir = HeapAlloc( GetProcessHeap(), 0, wcslen(home_dir) * sizeof(WCHAR) + sizeof(L"\\.cache") ))) return NULL;
         lstrcpyW( cache_dir, home_dir );
         lstrcatW( cache_dir, L"\\.cache" );
         cache_dir[1] = '\\';  /* change \??\ into \\?\ */
@@ -351,24 +339,24 @@ static WCHAR *get_cache_file_name(BOOL ensure_exists)
     if (ensure_exists && !CreateDirectoryW( cache_dir, NULL ) && GetLastError() != ERROR_ALREADY_EXISTS)
     {
         WARN( "%s does not exist and could not be created (%lu)\n", debugstr_w(cache_dir), GetLastError() );
-        heap_free( cache_dir );
+        HeapFree( GetProcessHeap(), 0, cache_dir );
         return NULL;
     }
 
     size = lstrlenW( cache_dir ) + ARRAY_SIZE(L"\\wine") + lstrlenW( addon->file_name ) + 1;
-    if (!(ret = heap_alloc( size * sizeof(WCHAR) )))
+    if (!(ret = malloc( size * sizeof(WCHAR) )))
     {
-        heap_free( cache_dir );
+        HeapFree( GetProcessHeap(), 0, cache_dir );
         return NULL;
     }
     lstrcpyW( ret, cache_dir );
     lstrcatW( ret, L"\\wine" );
-    heap_free( cache_dir );
+    HeapFree( GetProcessHeap(), 0, cache_dir );
 
     if (ensure_exists && !CreateDirectoryW( ret, NULL ) && GetLastError() != ERROR_ALREADY_EXISTS)
     {
         WARN( "%s does not exist and could not be created (%lu)\n", debugstr_w(ret), GetLastError() );
-        heap_free( ret );
+        free( ret );
         return NULL;
     }
     len = lstrlenW( ret );
@@ -391,12 +379,12 @@ static enum install_res install_from_cache(void)
     if(!sha_check(cache_file_name)) {
         WARN("could not validate checksum\n");
         DeleteFileW(cache_file_name);
-        heap_free(cache_file_name);
+        free(cache_file_name);
         return INSTALL_NEXT;
     }
 
     res = install_file(cache_file_name);
-    heap_free(cache_file_name);
+    free(cache_file_name);
     return res;
 }
 
@@ -497,7 +485,7 @@ static HRESULT WINAPI InstallCallback_OnStopBinding(IBindStatusCallback *iface,
         cache_file_name = get_cache_file_name(TRUE);
         if(cache_file_name) {
             CopyFileW(msi_file, cache_file_name, FALSE);
-            heap_free(cache_file_name);
+            free(cache_file_name);
         }
     }else {
         WCHAR message[256];
@@ -507,7 +495,7 @@ static HRESULT WINAPI InstallCallback_OnStopBinding(IBindStatusCallback *iface,
     }
 
     DeleteFileW(msi_file);
-    heap_free(msi_file);
+    free(msi_file);
     msi_file = NULL;
 
     EndDialog(install_dialog, 0);
@@ -527,7 +515,7 @@ static HRESULT WINAPI InstallCallback_OnDataAvailable(IBindStatusCallback *iface
         DWORD dwSize, FORMATETC* pformatetc, STGMEDIUM* pstgmed)
 {
     if(!msi_file) {
-        msi_file = heap_strdupW(pstgmed->u.lpszFileName);
+        msi_file = wcsdup(pstgmed->lpszFileName);
         TRACE("got file name %s\n", debugstr_w(msi_file));
     }
 
@@ -621,7 +609,7 @@ static void append_url_params( WCHAR *url )
     len += MultiByteToWideChar(CP_ACP, 0, addon->version, -1, url+len, size/sizeof(WCHAR)-len)-1;
     lstrcpyW(url+len, L"&winev=");
     len += lstrlenW(L"&winev=");
-    MultiByteToWideChar(CP_ACP, 0, wine_get_version(), -1, url+len, size/sizeof(WCHAR)-len);
+    MultiByteToWideChar(CP_ACP, 0, p_wine_get_version() ? p_wine_get_version() : 0, -1, url+len, size/sizeof(WCHAR)-len);
 }
 
 static LPWSTR get_url(void)
@@ -634,15 +622,15 @@ static LPWSTR get_url(void)
 
     static const WCHAR httpW[] = {'h','t','t','p'};
 
-    url = heap_alloc(size);
+    url = malloc(size);
     returned_size = size;
 
     hkey = open_config_key();
     if (hkey)
     {
-        config_key = heap_strdupAtoW(addon->url_config_key);
+        config_key = strdupAtoW(addon->url_config_key);
         res = RegQueryValueExW(hkey, config_key, NULL, &type, (LPBYTE)url, &returned_size);
-        heap_free(config_key);
+        free(config_key);
         RegCloseKey(hkey);
         if(res == ERROR_SUCCESS && type == REG_SZ) goto found;
     }
@@ -696,7 +684,7 @@ static void run_winebrowser(const WCHAR *url)
     lstrcpyW(app+len, L"\\winebrowser.exe");
     len += ARRAY_SIZE(L"\\winebrowser.exe") - 1;
 
-    args = heap_alloc((len+1+url_len)*sizeof(WCHAR));
+    args = malloc((len + 1 + url_len) * sizeof(WCHAR));
     if(!args)
         return;
 
@@ -709,7 +697,7 @@ static void run_winebrowser(const WCHAR *url)
     memset(&si, 0, sizeof(si));
     si.cb = sizeof(si);
     ret = CreateProcessW(app, args, NULL, NULL, FALSE, DETACHED_PROCESS, NULL, NULL, &si, &pi);
-    heap_free(args);
+    free(args);
     if (ret) {
         CloseHandle(pi.hThread);
         CloseHandle(pi.hProcess);
@@ -762,6 +750,7 @@ BOOL install_addon(addon_t addon_type)
     addon = addons_info+addon_type;
 
     p_wine_get_dos_file_name = (void *)GetProcAddress(GetModuleHandleW(L"kernel32.dll"), "wine_get_dos_file_name");
+    p_wine_get_version = (void *)GetProcAddress(GetModuleHandleW(L"ntdll.dll"), "wine_get_version");
 
     /*
      * Try to find addon .msi file in following order:
@@ -778,7 +767,7 @@ BOOL install_addon(addon_t addon_type)
         && (url = get_url()))
         DialogBoxW(hInst, addon->dialog_template, 0, installer_proc);
 
-    heap_free(url);
+    free(url);
     url = NULL;
     return TRUE;
 }
