@@ -45,7 +45,7 @@ struct path_list_node_t {
 /* init list */
 static BOOL init_path_list(path_list_node_t **node, REAL x, REAL y)
 {
-    *node = heap_alloc_zero(sizeof(path_list_node_t));
+    *node = calloc(1, sizeof(path_list_node_t));
     if(!*node)
         return FALSE;
 
@@ -64,7 +64,7 @@ static void free_path_list(path_list_node_t *node)
 
     while(n){
         n = n->next;
-        heap_free(node);
+        free(node);
         node = n;
     }
 }
@@ -79,7 +79,7 @@ static path_list_node_t* add_path_list_node(path_list_node_t *node, REAL x, REAL
 {
     path_list_node_t *new;
 
-    new = heap_alloc_zero(sizeof(path_list_node_t));
+    new = calloc(1, sizeof(path_list_node_t));
     if(!new)
         return NULL;
 
@@ -103,6 +103,34 @@ static INT path_list_count(path_list_node_t *node)
     return count;
 }
 
+struct flatten_bezier_job
+{
+    path_list_node_t *start;
+    REAL x2;
+    REAL y2;
+    REAL x3;
+    REAL y3;
+    path_list_node_t *end;
+    struct list entry;
+};
+
+static BOOL flatten_bezier_add(struct list *jobs, path_list_node_t *start, REAL x2, REAL y2, REAL x3, REAL y3, path_list_node_t *end)
+{
+    struct flatten_bezier_job *job = malloc(sizeof(struct flatten_bezier_job));
+    if (!job)
+        return FALSE;
+
+    job->start = start;
+    job->x2 = x2;
+    job->y2 = y2;
+    job->x3 = x3;
+    job->y3 = y3;
+    job->end = end;
+
+    list_add_after(jobs, &job->entry);
+    return TRUE;
+}
+
 /* GdipFlattenPath helper */
 /*
  * Used to recursively flatten single Bezier curve
@@ -127,45 +155,85 @@ static BOOL flatten_bezier(path_list_node_t *start, REAL x2, REAL y2, REAL x3, R
     GpPointF mp[5];
     GpPointF pt, pt_st;
     path_list_node_t *node;
+    REAL area_triangle, distance_start_end;
+    BOOL ret = TRUE;
+    struct list jobs;
+    struct flatten_bezier_job *current, *next;
 
-    /* calculate bezier curve middle points == new control points */
-    mp[0].X = (start->pt.X + x2) / 2.0;
-    mp[0].Y = (start->pt.Y + y2) / 2.0;
-    /* middle point between control points */
-    pt.X = (x2 + x3) / 2.0;
-    pt.Y = (y2 + y3) / 2.0;
-    mp[1].X = (mp[0].X + pt.X) / 2.0;
-    mp[1].Y = (mp[0].Y + pt.Y) / 2.0;
-    mp[4].X = (end->pt.X + x3) / 2.0;
-    mp[4].Y = (end->pt.Y + y3) / 2.0;
-    mp[3].X = (mp[4].X + pt.X) / 2.0;
-    mp[3].Y = (mp[4].Y + pt.Y) / 2.0;
+    list_init( &jobs );
+    flatten_bezier_add(&jobs, start, x2, y2, x3, y3, end);
+    LIST_FOR_EACH_ENTRY( current, &jobs, struct flatten_bezier_job, entry )
+    {
+        start = current->start;
+        x2 = current->x2;
+        y2 = current->y2;
+        x3 = current->x3;
+        y3 = current->y3;
+        end = current->end;
 
-    mp[2].X = (mp[1].X + mp[3].X) / 2.0;
-    mp[2].Y = (mp[1].Y + mp[3].Y) / 2.0;
+        /* middle point between control points */
+        pt.X = (x2 + x3) / 2.0;
+        pt.Y = (y2 + y3) / 2.0;
 
-    if ((x2 == mp[0].X && y2 == mp[0].Y && x3 == mp[1].X && y3 == mp[1].Y) ||
-        (x2 == mp[3].X && y2 == mp[3].Y && x3 == mp[4].X && y3 == mp[4].Y))
-        return TRUE;
+        /* calculate bezier curve middle points == new control points */
+        mp[0].X = (start->pt.X + x2) / 2.0;
+        mp[0].Y = (start->pt.Y + y2) / 2.0;
+        mp[1].X = (mp[0].X + pt.X) / 2.0;
+        mp[1].Y = (mp[0].Y + pt.Y) / 2.0;
+        mp[4].X = (end->pt.X + x3) / 2.0;
+        mp[4].Y = (end->pt.Y + y3) / 2.0;
+        mp[3].X = (mp[4].X + pt.X) / 2.0;
+        mp[3].Y = (mp[4].Y + pt.Y) / 2.0;
 
-    pt = end->pt;
-    pt_st = start->pt;
-    /* check flatness as a half of distance between middle point and a linearized path */
-    if(fabs(((pt.Y - pt_st.Y)*mp[2].X + (pt_st.X - pt.X)*mp[2].Y +
-        (pt_st.Y*pt.X - pt_st.X*pt.Y))) <=
-        (0.5 * flatness*sqrtf((powf(pt.Y - pt_st.Y, 2.0) + powf(pt_st.X - pt.X, 2.0))))){
-        return TRUE;
+        /* middle point between new control points */
+        mp[2].X = (mp[1].X + mp[3].X) / 2.0;
+        mp[2].Y = (mp[1].Y + mp[3].Y) / 2.0;
+
+        pt = end->pt;
+        pt_st = start->pt;
+
+        /* Test for closely spaced points that don't need to be flattened
+         * Also avoids limited-precision errors in flatness check
+         */
+        if((fabs(pt.X - mp[2].X) + fabs(pt.Y - mp[2].Y) +
+            fabs(pt_st.X - mp[2].X) + fabs(pt_st.Y - mp[2].Y) ) <= flatness * 0.5)
+            continue;
+
+        /* check flatness as a half of distance between middle point and a linearized path
+         * formula for distance point from line for point (x0, y0) and line (x1, y1) (x2, y2)
+         * is defined as (area_triangle / distance_start_end):
+         *     | (x2 - x1) * (y1 - y0) - (x1 - x0) * (y2 - y1) / sqrt( (x2 - x1)^2 + (y2 - y1)^2 ) |
+         * Here rearranged to avoid division and simplified:
+         *     x0(y2 - y1) + y0(x1 - x2) + (x2*y1 - x1*y2)
+         */
+        area_triangle = (pt.Y - pt_st.Y)*mp[2].X + (pt_st.X - pt.X)*mp[2].Y + (pt_st.Y*pt.X - pt_st.X*pt.Y);
+        distance_start_end = hypotf(pt.Y - pt_st.Y, pt_st.X - pt.X);
+        if(fabs(area_triangle) <= (0.5 * flatness * distance_start_end)){
+            continue;
+        }
+        else
+            /* add a middle point */
+            if(!(node = add_path_list_node(start, mp[2].X, mp[2].Y, PathPointTypeLine)))
+            {
+                ret = FALSE;
+                break;
+            };
+
+        /* do the same with halves */
+        if (!flatten_bezier_add(&current->entry, node,  mp[3].X, mp[3].Y, mp[4].X, mp[4].Y, end))
+            break;
+        if (!flatten_bezier_add(&current->entry, start, mp[0].X, mp[0].Y, mp[1].X, mp[1].Y, node))
+            break;
     }
-    else
-        /* add a middle point */
-        if(!(node = add_path_list_node(start, mp[2].X, mp[2].Y, PathPointTypeLine)))
-            return FALSE;
 
-    /* do the same with halves */
-    flatten_bezier(start, mp[0].X, mp[0].Y, mp[1].X, mp[1].Y, node, flatness);
-    flatten_bezier(node,  mp[3].X, mp[3].Y, mp[4].X, mp[4].Y, end,  flatness);
+    /* Cleanup */
+    LIST_FOR_EACH_ENTRY_SAFE( current, next, &jobs, struct flatten_bezier_job, entry )
+    {
+        list_remove(&current->entry);
+        free(current);
+    }
 
-    return TRUE;
+    return ret;
 }
 
 /* GdipAddPath* helper
@@ -221,10 +289,10 @@ static GpStatus extend_current_figure(GpPath *path, GDIPCONST PointF *points, IN
  *
  * PARAMS
  *  path       [I/O] Path that the arc is appended to
- *  x1         [I]   X coordinate of the boundary box
- *  y1         [I]   Y coordinate of the boundary box
- *  x2         [I]   Width of the boundary box
- *  y2         [I]   Height of the boundary box
+ *  x          [I]   X coordinate of the boundary rectangle
+ *  y          [I]   Y coordinate of the boundary rectangle
+ *  width      [I]   Width of the boundary rectangle
+ *  height     [I]   Height of the boundary rectangle
  *  startAngle [I]   Starting angle of the arc, clockwise
  *  sweepAngle [I]   Angle of the arc, clockwise
  *
@@ -240,32 +308,32 @@ static GpStatus extend_current_figure(GpPath *path, GDIPCONST PointF *points, IN
  *  In both cases, the value of newfigure of the given path is FALSE
  *  afterwards.
  */
-GpStatus WINGDIPAPI GdipAddPathArc(GpPath *path, REAL x1, REAL y1, REAL x2,
-    REAL y2, REAL startAngle, REAL sweepAngle)
+GpStatus WINGDIPAPI GdipAddPathArc(GpPath *path, REAL x, REAL y, REAL width,
+    REAL height, REAL startAngle, REAL sweepAngle)
 {
     GpPointF *points;
     GpStatus status;
     INT count;
 
     TRACE("(%p, %.2f, %.2f, %.2f, %.2f, %.2f, %.2f)\n",
-          path, x1, y1, x2, y2, startAngle, sweepAngle);
+          path, x, y, width, height, startAngle, sweepAngle);
 
-    if(!path)
+    if(!path || width <= 0.0f || height <= 0.0f)
         return InvalidParameter;
 
-    count = arc2polybezier(NULL, x1, y1, x2, y2, startAngle, sweepAngle);
+    count = arc2polybezier(NULL, x, y, width, height, startAngle, sweepAngle);
     if(count == 0)
         return Ok;
 
-    points = heap_alloc_zero(sizeof(GpPointF)*count);
+    points = malloc(sizeof(GpPointF) * count);
     if(!points)
         return OutOfMemory;
 
-    arc2polybezier(points, x1, y1, x2, y2, startAngle, sweepAngle);
+    arc2polybezier(points, x, y, width, height, startAngle, sweepAngle);
 
     status = extend_current_figure(path, points, count, PathPointTypeBezier);
 
-    heap_free(points);
+    free(points);
     return status;
 }
 
@@ -339,7 +407,7 @@ GpStatus WINGDIPAPI GdipAddPathBeziersI(GpPath *path, GDIPCONST GpPoint *points,
     if(!points || ((count - 1) % 3))
         return InvalidParameter;
 
-    ptsF = heap_alloc_zero(sizeof(GpPointF) * count);
+    ptsF = malloc(sizeof(GpPointF) * count);
     if(!ptsF)
         return OutOfMemory;
 
@@ -349,7 +417,7 @@ GpStatus WINGDIPAPI GdipAddPathBeziersI(GpPath *path, GDIPCONST GpPoint *points,
     }
 
     ret = GdipAddPathBeziers(path, ptsF, count);
-    heap_free(ptsF);
+    free(ptsF);
 
     return ret;
 }
@@ -359,7 +427,7 @@ GpStatus WINGDIPAPI GdipAddPathClosedCurve(GpPath *path, GDIPCONST GpPointF *poi
 {
     TRACE("(%p, %p, %d)\n", path, points, count);
 
-    return GdipAddPathClosedCurve2(path, points, count, 1.0);
+    return GdipAddPathClosedCurve2(path, points, count, 0.5);
 }
 
 GpStatus WINGDIPAPI GdipAddPathClosedCurveI(GpPath *path, GDIPCONST GpPoint *points,
@@ -367,7 +435,7 @@ GpStatus WINGDIPAPI GdipAddPathClosedCurveI(GpPath *path, GDIPCONST GpPoint *poi
 {
     TRACE("(%p, %p, %d)\n", path, points, count);
 
-    return GdipAddPathClosedCurve2I(path, points, count, 1.0);
+    return GdipAddPathClosedCurve2I(path, points, count, 0.5);
 }
 
 GpStatus WINGDIPAPI GdipAddPathClosedCurve2(GpPath *path, GDIPCONST GpPointF *points,
@@ -384,11 +452,11 @@ GpStatus WINGDIPAPI GdipAddPathClosedCurve2(GpPath *path, GDIPCONST GpPointF *po
     if(!path || !points || count <= 1)
         return InvalidParameter;
 
-    pt = heap_alloc_zero(len_pt * sizeof(GpPointF));
-    pts = heap_alloc_zero((count + 1)*sizeof(GpPointF));
+    pt = malloc(len_pt * sizeof(GpPointF));
+    pts = malloc((count + 1) * sizeof(GpPointF));
     if(!pt || !pts){
-        heap_free(pt);
-        heap_free(pts);
+        free(pt);
+        free(pts);
         return OutOfMemory;
     }
 
@@ -434,8 +502,8 @@ GpStatus WINGDIPAPI GdipAddPathClosedCurve2(GpPath *path, GDIPCONST GpPointF *po
         path->newfigure = TRUE;
     }
 
-    heap_free(pts);
-    heap_free(pt);
+    free(pts);
+    free(pt);
 
     return stat;
 }
@@ -452,7 +520,7 @@ GpStatus WINGDIPAPI GdipAddPathClosedCurve2I(GpPath *path, GDIPCONST GpPoint *po
     if(!path || !points || count <= 1)
         return InvalidParameter;
 
-    ptf = heap_alloc_zero(sizeof(GpPointF)*count);
+    ptf = malloc(sizeof(GpPointF) * count);
     if(!ptf)
         return OutOfMemory;
 
@@ -463,7 +531,7 @@ GpStatus WINGDIPAPI GdipAddPathClosedCurve2I(GpPath *path, GDIPCONST GpPoint *po
 
     stat = GdipAddPathClosedCurve2(path, ptf, count, tension);
 
-    heap_free(ptf);
+    free(ptf);
 
     return stat;
 }
@@ -472,71 +540,76 @@ GpStatus WINGDIPAPI GdipAddPathCurve(GpPath *path, GDIPCONST GpPointF *points, I
 {
     TRACE("(%p, %p, %d)\n", path, points, count);
 
-    if(!path || !points || count <= 1)
-        return InvalidParameter;
-
-    return GdipAddPathCurve2(path, points, count, 1.0);
+    return GdipAddPathCurve3(path, points, count, 0, count - 1, 0.5);
 }
 
 GpStatus WINGDIPAPI GdipAddPathCurveI(GpPath *path, GDIPCONST GpPoint *points, INT count)
 {
     TRACE("(%p, %p, %d)\n", path, points, count);
 
-    if(!path || !points || count <= 1)
-        return InvalidParameter;
-
-    return GdipAddPathCurve2I(path, points, count, 1.0);
+    return GdipAddPathCurve3I(path, points, count, 0, count - 1, 0.5);
 }
 
-GpStatus WINGDIPAPI GdipAddPathCurve2(GpPath *path, GDIPCONST GpPointF *points, INT count,
-    REAL tension)
+GpStatus WINGDIPAPI GdipAddPathCurve3(GpPath *path, GDIPCONST GpPointF *points,
+    INT count, INT offset, INT nseg, REAL tension)
 {
-    INT i, len_pt = count*3-2;
+    INT i, len_pt = nseg * 3 + 1;
     GpPointF *pt;
     REAL x1, x2, y1, y2;
     GpStatus stat;
+    TRACE("(%p, %p, %d, %d, %d, %.2f)\n", path, points, count, offset, nseg, tension);
 
-    TRACE("(%p, %p, %d, %.2f)\n", path, points, count, tension);
-
-    if(!path || !points || count <= 1)
+    if(!path || !points || offset + 1 >= count || count - offset < nseg + 1 || nseg < 1)
         return InvalidParameter;
 
-    pt = heap_alloc_zero(len_pt * sizeof(GpPointF));
+    pt = calloc(len_pt, sizeof(GpPointF));
     if(!pt)
         return OutOfMemory;
 
     tension = tension * TENSION_CONST;
 
-    calc_curve_bezier_endp(points[0].X, points[0].Y, points[1].X, points[1].Y,
-        tension, &x1, &y1);
+    pt[0].X = points[offset].X;
+    pt[0].Y = points[offset].Y;
+    if (offset > 0)
+    {
+        calc_curve_bezier(&(points[offset - 1]), tension, &x1, &y1, &x2, &y2);
+        pt[1].X = x2;
+        pt[1].Y = y2;
+    }
+    else
+    {
+        calc_curve_bezier_endp(points[offset].X, points[offset].Y,
+            points[offset + 1].X, points[offset + 1].Y, tension, &x1, &y1);
+        pt[1].X = x1;
+        pt[1].Y = y1;
+    }
 
-    pt[0].X = points[0].X;
-    pt[0].Y = points[0].Y;
-    pt[1].X = x1;
-    pt[1].Y = y1;
-
-    for(i = 0; i < count-2; i++){
-        calc_curve_bezier(&(points[i]), tension, &x1, &y1, &x2, &y2);
+    for (i = 0; i < nseg - 1; i++){
+        calc_curve_bezier(&(points[offset + i]), tension, &x1, &y1, &x2, &y2);
 
         pt[3*i+2].X = x1;
         pt[3*i+2].Y = y1;
-        pt[3*i+3].X = points[i+1].X;
-        pt[3*i+3].Y = points[i+1].Y;
+        pt[3*i+3].X = points[offset + i + 1].X;
+        pt[3*i+3].Y = points[offset + i + 1].Y;
         pt[3*i+4].X = x2;
         pt[3*i+4].Y = y2;
     }
 
-    calc_curve_bezier_endp(points[count-1].X, points[count-1].Y,
-        points[count-2].X, points[count-2].Y, tension, &x1, &y1);
+    if (offset + nseg + 1 < count)
+        /* If there are one more point in points table then use it for curve calculation */
+        calc_curve_bezier(&(points[offset + nseg - 1]), tension, &x1, &y1, &x2, &y2);
+    else
+        calc_curve_bezier_endp(points[offset + nseg].X, points[offset + nseg].Y,
+            points[offset + nseg - 1].X, points[offset + nseg - 1].Y, tension, &x1, &y1);
 
     pt[len_pt-2].X = x1;
     pt[len_pt-2].Y = y1;
-    pt[len_pt-1].X = points[count-1].X;
-    pt[len_pt-1].Y = points[count-1].Y;
+    pt[len_pt-1].X = points[offset + nseg].X;
+    pt[len_pt-1].Y = points[offset + nseg].Y;
 
     stat = extend_current_figure(path, pt, len_pt, PathPointTypeBezier);
 
-    heap_free(pt);
+    free(pt);
 
     return stat;
 }
@@ -544,51 +617,45 @@ GpStatus WINGDIPAPI GdipAddPathCurve2(GpPath *path, GDIPCONST GpPointF *points,
 GpStatus WINGDIPAPI GdipAddPathCurve2I(GpPath *path, GDIPCONST GpPoint *points,
     INT count, REAL tension)
 {
-    GpPointF *ptf;
-    INT i;
-    GpStatus stat;
-
     TRACE("(%p, %p, %d, %.2f)\n", path, points, count, tension);
 
-    if(!path || !points || count <= 1)
-        return InvalidParameter;
-
-    ptf = heap_alloc_zero(sizeof(GpPointF)*count);
-    if(!ptf)
-        return OutOfMemory;
-
-    for(i = 0; i < count; i++){
-        ptf[i].X = (REAL)points[i].X;
-        ptf[i].Y = (REAL)points[i].Y;
-    }
-
-    stat = GdipAddPathCurve2(path, ptf, count, tension);
-
-    heap_free(ptf);
-
-    return stat;
+    return GdipAddPathCurve3I(path, points, count, 0, count - 1, tension);
 }
 
-GpStatus WINGDIPAPI GdipAddPathCurve3(GpPath *path, GDIPCONST GpPointF *points,
-    INT count, INT offset, INT nseg, REAL tension)
+GpStatus WINGDIPAPI GdipAddPathCurve2(GpPath *path, GDIPCONST GpPointF *points, INT count,
+    REAL tension)
 {
-    TRACE("(%p, %p, %d, %d, %d, %.2f)\n", path, points, count, offset, nseg, tension);
+    TRACE("(%p, %p, %d, %.2f)\n", path, points, count, tension);
 
-    if(!path || !points || offset + 1 >= count || count - offset < nseg + 1)
-        return InvalidParameter;
-
-    return GdipAddPathCurve2(path, &points[offset], nseg + 1, tension);
+    return GdipAddPathCurve3(path, points, count, 0, count - 1, tension);
 }
 
 GpStatus WINGDIPAPI GdipAddPathCurve3I(GpPath *path, GDIPCONST GpPoint *points,
     INT count, INT offset, INT nseg, REAL tension)
 {
+    GpPointF *ptf;
+    INT i;
+    GpStatus stat;
+
     TRACE("(%p, %p, %d, %d, %d, %.2f)\n", path, points, count, offset, nseg, tension);
 
-    if(!path || !points || offset + 1 >= count || count - offset < nseg + 1)
+    if(!path || !points || offset + 1 >= count || count - offset < nseg + 1 || nseg < 1)
         return InvalidParameter;
 
-    return GdipAddPathCurve2I(path, &points[offset], nseg + 1, tension);
+    ptf = malloc(sizeof(GpPointF) * count);
+    if(!ptf)
+        return OutOfMemory;
+
+    for(i = 0; i < count; i++) {
+        ptf[i].X = (REAL)points[i].X;
+        ptf[i].Y = (REAL)points[i].Y;
+    }
+
+    stat = GdipAddPathCurve3(path, ptf, count, offset, nseg, tension);
+
+    free(ptf);
+
+    return stat;
 }
 
 GpStatus WINGDIPAPI GdipAddPathEllipse(GpPath *path, REAL x, REAL y, REAL width,
@@ -653,8 +720,8 @@ GpStatus WINGDIPAPI GdipAddPathLine2I(GpPath *path, GDIPCONST GpPoint *points, I
     if(count <= 0)
         return InvalidParameter;
 
-    pointsF = heap_alloc_zero(sizeof(GpPointF) * count);
-    if(!pointsF)    return OutOfMemory;
+    pointsF = malloc(sizeof(GpPointF) * count);
+    if(!pointsF) return OutOfMemory;
 
     for(i = 0;i < count; i++){
         pointsF[i].X = (REAL)points[i].X;
@@ -663,7 +730,7 @@ GpStatus WINGDIPAPI GdipAddPathLine2I(GpPath *path, GDIPCONST GpPoint *points, I
 
     stat = GdipAddPathLine2(path, pointsF, count);
 
-    heap_free(pointsF);
+    free(pointsF);
 
     return stat;
 }
@@ -781,7 +848,7 @@ GpStatus WINGDIPAPI GdipAddPathPie(GpPath *path, REAL x, REAL y, REAL width, REA
     if(count == 0)
         return Ok;
 
-    ptf = heap_alloc_zero(sizeof(GpPointF)*count);
+    ptf = malloc(sizeof(GpPointF) * count);
     if(!ptf)
         return OutOfMemory;
 
@@ -789,12 +856,12 @@ GpStatus WINGDIPAPI GdipAddPathPie(GpPath *path, REAL x, REAL y, REAL width, REA
 
     status = GdipAddPathLine(path, x + width/2, y + height/2, ptf[0].X, ptf[0].Y);
     if(status != Ok){
-        heap_free(ptf);
+        free(ptf);
         return status;
     }
     /* one spline is already added as a line endpoint */
     if(!lengthen_path(path, count - 1)){
-        heap_free(ptf);
+        free(ptf);
         return OutOfMemory;
     }
 
@@ -806,7 +873,7 @@ GpStatus WINGDIPAPI GdipAddPathPie(GpPath *path, REAL x, REAL y, REAL width, REA
 
     GdipClosePathFigure(path);
 
-    heap_free(ptf);
+    free(ptf);
 
     return status;
 }
@@ -857,7 +924,7 @@ GpStatus WINGDIPAPI GdipAddPathPolygonI(GpPath *path, GDIPCONST GpPoint *points,
     if(!points || count < 3)
         return InvalidParameter;
 
-    ptf = heap_alloc_zero(sizeof(GpPointF) * count);
+    ptf = malloc(sizeof(GpPointF) * count);
     if(!ptf)
         return OutOfMemory;
 
@@ -868,7 +935,7 @@ GpStatus WINGDIPAPI GdipAddPathPolygonI(GpPath *path, GDIPCONST GpPoint *points,
 
     status = GdipAddPathPolygon(path, ptf, count);
 
-    heap_free(ptf);
+    free(ptf);
 
     return status;
 }
@@ -908,7 +975,7 @@ static GpStatus format_string_callback(HDC dc,
     if (y + bounds->Height * args->scale > args->maxY)
         args->maxY = y + bounds->Height * args->scale;
 
-    for (i = index; i < length; ++i)
+    for (i = index; i < length + index; ++i)
     {
         GLYPHMETRICS gm;
         TTPOLYGONHEADER *ph = NULL, *origph;
@@ -920,11 +987,11 @@ static GpStatus format_string_callback(HDC dc,
             status = GenericError;
             break;
         }
-        origph = ph = heap_alloc_zero(len);
+        origph = ph = calloc(1, len);
         start = (char *)ph;
         if (!ph || !lengthen_path(path, len / sizeof(POINTFX)))
         {
-            heap_free(ph);
+            free(ph);
             status = OutOfMemory;
             break;
         }
@@ -975,7 +1042,7 @@ static GpStatus format_string_callback(HDC dc,
         x += gm.gmCellIncX * args->scale;
         y += gm.gmCellIncY * args->scale;
 
-        heap_free(origph);
+        free(origph);
         if (status != Ok)
             break;
     }
@@ -999,9 +1066,12 @@ GpStatus WINGDIPAPI GdipAddPathString(GpPath* path, GDIPCONST WCHAR* string, INT
     TEXTMETRICW textmetric;
 
     TRACE("(%p, %s, %d, %p, %d, %f, %p, %p)\n", path, debugstr_w(string), length, family, style, emSize, layoutRect, format);
-    if (!path || !string || !family || !emSize || !layoutRect || !format)
+    if (!path || !string || !family || !emSize || !layoutRect)
         return InvalidParameter;
 
+    if (!format)
+        format = &default_drawstring_format;
+
     status = GdipGetEmHeight(family, style, &native_height);
     if (status != Ok)
         return status;
@@ -1061,10 +1131,10 @@ GpStatus WINGDIPAPI GdipAddPathString(GpPath* path, GDIPCONST WCHAR* string, INT
 
     if (status != Ok) /* free backup */
     {
-        heap_free(path->pathdata.Points);
-        heap_free(path->pathdata.Types);
+        free(path->pathdata.Points);
+        free(path->pathdata.Types);
         *path = *backup;
-        heap_free(backup);
+        free(backup);
         return status;
     }
     if (format->line_align == StringAlignmentCenter && layoutRect->Y + args.maxY < layoutRect->Height)
@@ -1115,17 +1185,17 @@ GpStatus WINGDIPAPI GdipClonePath(GpPath* path, GpPath **clone)
     if(!path || !clone)
         return InvalidParameter;
 
-    *clone = heap_alloc_zero(sizeof(GpPath));
+    *clone = malloc(sizeof(GpPath));
     if(!*clone) return OutOfMemory;
 
     **clone = *path;
 
-    (*clone)->pathdata.Points = heap_alloc_zero(path->datalen * sizeof(PointF));
-    (*clone)->pathdata.Types = heap_alloc_zero(path->datalen);
+    (*clone)->pathdata.Points = malloc(path->datalen * sizeof(PointF));
+    (*clone)->pathdata.Types = malloc(path->datalen);
     if(!(*clone)->pathdata.Points || !(*clone)->pathdata.Types){
-        heap_free((*clone)->pathdata.Points);
-        heap_free((*clone)->pathdata.Types);
-        heap_free(*clone);
+        free((*clone)->pathdata.Points);
+        free((*clone)->pathdata.Types);
+        free(*clone);
         return OutOfMemory;
     }
 
@@ -1177,7 +1247,7 @@ GpStatus WINGDIPAPI GdipCreatePath(GpFillMode fill, GpPath **path)
     if(!path)
         return InvalidParameter;
 
-    *path = heap_alloc_zero(sizeof(GpPath));
+    *path = calloc(1, sizeof(GpPath));
     if(!*path)  return OutOfMemory;
 
     (*path)->fill = fill;
@@ -1201,8 +1271,8 @@ GpStatus WINGDIPAPI GdipCreatePath2(GDIPCONST GpPointF* points,
         return OutOfMemory;
     }
 
-    *path = heap_alloc_zero(sizeof(GpPath));
-    if(!*path)  return OutOfMemory;
+    *path = calloc(1, sizeof(GpPath));
+    if(!*path) return OutOfMemory;
 
     if(count > 1 && (types[count-1] & PathPointTypePathTypeMask) == PathPointTypeStart)
         count = 0;
@@ -1220,13 +1290,13 @@ GpStatus WINGDIPAPI GdipCreatePath2(GDIPCONST GpPointF* points,
         }
     }
 
-    (*path)->pathdata.Points = heap_alloc_zero(count * sizeof(PointF));
-    (*path)->pathdata.Types = heap_alloc_zero(count);
+    (*path)->pathdata.Points = malloc(count * sizeof(PointF));
+    (*path)->pathdata.Types = malloc(count);
 
     if(!(*path)->pathdata.Points || !(*path)->pathdata.Types){
-        heap_free((*path)->pathdata.Points);
-        heap_free((*path)->pathdata.Types);
-        heap_free(*path);
+        free((*path)->pathdata.Points);
+        free((*path)->pathdata.Types);
+        free(*path);
         return OutOfMemory;
     }
 
@@ -1252,7 +1322,7 @@ GpStatus WINGDIPAPI GdipCreatePath2I(GDIPCONST GpPoint* points,
 
     TRACE("(%p, %p, %d, %d, %p)\n", points, types, count, fill, path);
 
-    ptF = heap_alloc_zero(sizeof(GpPointF)*count);
+    ptF = malloc(sizeof(GpPointF) * count);
 
     for(i = 0;i < count; i++){
         ptF[i].X = (REAL)points[i].X;
@@ -1261,7 +1331,7 @@ GpStatus WINGDIPAPI GdipCreatePath2I(GDIPCONST GpPoint* points,
 
     ret = GdipCreatePath2(ptF, types, count, fill, path);
 
-    heap_free(ptF);
+    free(ptF);
 
     return ret;
 }
@@ -1273,9 +1343,9 @@ GpStatus WINGDIPAPI GdipDeletePath(GpPath *path)
     if(!path)
         return InvalidParameter;
 
-    heap_free(path->pathdata.Points);
-    heap_free(path->pathdata.Types);
-    heap_free(path);
+    free(path->pathdata.Points);
+    free(path->pathdata.Types);
+    free(path);
 
     return Ok;
 }
@@ -1446,8 +1516,8 @@ GpStatus WINGDIPAPI GdipGetPathPointsI(GpPath *path, GpPoint* points, INT count)
     if(count <= 0)
         return InvalidParameter;
 
-    ptf = heap_alloc_zero(sizeof(GpPointF)*count);
-    if(!ptf)    return OutOfMemory;
+    ptf = malloc(sizeof(GpPointF) * count);
+    if(!ptf) return OutOfMemory;
 
     ret = GdipGetPathPoints(path,ptf,count);
     if(ret == Ok)
@@ -1455,7 +1525,7 @@ GpStatus WINGDIPAPI GdipGetPathPointsI(GpPath *path, GpPoint* points, INT count)
             points[i].X = gdip_round(ptf[i].X);
             points[i].Y = gdip_round(ptf[i].Y);
         };
-    heap_free(ptf);
+    free(ptf);
 
     return ret;
 }
@@ -1486,7 +1556,7 @@ GpStatus WINGDIPAPI GdipGetPathWorldBounds(GpPath* path, GpRectF* bounds,
     INT count, i;
     REAL path_width = 1.0, width, height, temp, low_x, low_y, high_x, high_y;
 
-    TRACE("(%p, %p, %p, %p)\n", path, bounds, matrix, pen);
+    TRACE("(%p, %p, %s, %p)\n", path, bounds, debugstr_matrix(matrix), pen);
 
     /* Matrix and pen can be null. */
     if(!path || !bounds)
@@ -1568,7 +1638,7 @@ GpStatus WINGDIPAPI GdipGetPathWorldBoundsI(GpPath* path, GpRect* bounds,
     GpStatus ret;
     GpRectF boundsF;
 
-    TRACE("(%p, %p, %p, %p)\n", path, bounds, matrix, pen);
+    TRACE("(%p, %p, %s, %p)\n", path, bounds, debugstr_matrix(matrix), pen);
 
     ret = GdipGetPathWorldBounds(path,&boundsF,matrix,pen);
 
@@ -1609,12 +1679,12 @@ GpStatus WINGDIPAPI GdipReversePath(GpPath* path)
 
     if(count == 0) return Ok;
 
-    revpath.Points = heap_alloc_zero(sizeof(GpPointF)*count);
-    revpath.Types  = heap_alloc_zero(sizeof(BYTE)*count);
+    revpath.Points = calloc(count, sizeof(GpPointF));
+    revpath.Types  = calloc(count, sizeof(BYTE));
     revpath.Count  = count;
     if(!revpath.Points || !revpath.Types){
-        heap_free(revpath.Points);
-        heap_free(revpath.Types);
+        free(revpath.Points);
+        free(revpath.Types);
         return OutOfMemory;
     }
 
@@ -1644,8 +1714,8 @@ GpStatus WINGDIPAPI GdipReversePath(GpPath* path)
     memcpy(path->pathdata.Points, revpath.Points, sizeof(GpPointF)*count);
     memcpy(path->pathdata.Types,  revpath.Types,  sizeof(BYTE)*count);
 
-    heap_free(revpath.Points);
-    heap_free(revpath.Types);
+    free(revpath.Points);
+    free(revpath.Types);
 
     return Ok;
 }
@@ -1663,9 +1733,10 @@ GpStatus WINGDIPAPI GdipIsOutlineVisiblePathPoint(GpPath* path, REAL x, REAL y,
 {
     GpStatus stat;
     GpPath *wide_path;
+    GpPointF pt = {x, y};
     GpMatrix *transform = NULL;
 
-    TRACE("(%p,%0.2f,%0.2f,%p,%p,%p)\n", path, x, y, pen, graphics, result);
+    TRACE("(%p, %0.2f, %0.2f, %p, %p, %p)\n", path, x, y, pen, graphics, result);
 
     if(!path || !pen)
         return InvalidParameter;
@@ -1682,22 +1753,15 @@ GpStatus WINGDIPAPI GdipIsOutlineVisiblePathPoint(GpPath* path, REAL x, REAL y,
         if (stat == Ok)
             stat = get_graphics_transform(graphics, CoordinateSpaceDevice,
                 CoordinateSpaceWorld, transform);
+        if (stat == Ok)
+            GdipTransformMatrixPoints(transform, &pt, 1);
     }
 
     if (stat == Ok)
-        stat = GdipWidenPath(wide_path, pen, transform, 1.0);
-
-    if (pen->unit == UnitPixel && graphics != NULL)
-    {
-        if (stat == Ok)
-            stat = GdipInvertMatrix(transform);
-
-        if (stat == Ok)
-            stat = GdipTransformPath(wide_path, transform);
-    }
+        stat = GdipWidenPath(wide_path, pen, transform, 0.25f);
 
     if (stat == Ok)
-        stat = GdipIsVisiblePathPoint(wide_path, x, y, graphics, result);
+        stat = GdipIsVisiblePathPoint(wide_path, pt.X, pt.Y, graphics, result);
 
     GdipDeleteMatrix(transform);
 
@@ -1728,7 +1792,7 @@ GpStatus WINGDIPAPI GdipIsVisiblePathPoint(GpPath* path, REAL x, REAL y, GpGraph
     if(status != Ok)
         return status;
 
-    status = GdipGetRegionHRgn(region, graphics, &hrgn);
+    status = GdipGetRegionHRgn(region, NULL, &hrgn);
     if(status != Ok){
         GdipDeleteRegion(region);
         return status;
@@ -1782,7 +1846,7 @@ GpStatus WINGDIPAPI GdipSetPathFillMode(GpPath *path, GpFillMode fill)
 
 GpStatus WINGDIPAPI GdipTransformPath(GpPath *path, GpMatrix *matrix)
 {
-    TRACE("(%p, %p)\n", path, matrix);
+    TRACE("(%p, %s)\n", path, debugstr_matrix(matrix));
 
     if(!path)
         return InvalidParameter;
@@ -1798,7 +1862,7 @@ GpStatus WINGDIPAPI GdipWarpPath(GpPath *path, GpMatrix* matrix,
     GDIPCONST GpPointF *points, INT count, REAL x, REAL y, REAL width,
     REAL height, WarpMode warpmode, REAL flatness)
 {
-    FIXME("(%p,%p,%p,%i,%0.2f,%0.2f,%0.2f,%0.2f,%i,%0.2f)\n", path, matrix,
+    FIXME("(%p,%s,%p,%i,%0.2f,%0.2f,%0.2f,%0.2f,%i,%0.2f)\n", path, debugstr_matrix(matrix),
         points, count, x, y, width, height, warpmode, flatness);
 
     return NotImplemented;
@@ -1809,7 +1873,7 @@ static void add_bevel_point(const GpPointF *endpoint, const GpPointF *nextpoint,
 {
     REAL segment_dy = nextpoint->Y-endpoint->Y;
     REAL segment_dx = nextpoint->X-endpoint->X;
-    REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+    REAL segment_length = hypotf(segment_dy, segment_dx);
     REAL distance = pen_width / 2.0;
     REAL bevel_dx, bevel_dy;
 
@@ -1845,8 +1909,8 @@ static void widen_joint(const GpPointF *p1, const GpPointF *p2, const GpPointF *
         if ((p2->X - p1->X) * (p3->Y - p1->Y) > (p2->Y - p1->Y) * (p3->X - p1->X))
         {
             float distance = pen_width / 2.0;
-            float length_0 = sqrtf((p2->X-p1->X)*(p2->X-p1->X)+(p2->Y-p1->Y)*(p2->Y-p1->Y));
-            float length_1 = sqrtf((p3->X-p2->X)*(p3->X-p2->X)+(p3->Y-p2->Y)*(p3->Y-p2->Y));
+            float length_0 = hypotf(p2->X - p1->X, p2->Y - p1->Y);
+            float length_1 = hypotf(p3->X - p2->X, p3->Y - p2->Y);
             float dx0 = distance * (p2->X - p1->X) / length_0;
             float dy0 = distance * (p2->Y - p1->Y) / length_0;
             float dx1 = distance * (p3->X - p2->X) / length_1;
@@ -1896,7 +1960,7 @@ static void widen_cap(const GpPointF *endpoint, const GpPointF *nextpoint,
     {
         REAL segment_dy = nextpoint->Y-endpoint->Y;
         REAL segment_dx = nextpoint->X-endpoint->X;
-        REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+        REAL segment_length = hypotf(segment_dy, segment_dx);
         REAL distance = pen_width / 2.0;
         REAL bevel_dx, bevel_dy;
         REAL extend_dx, extend_dy;
@@ -1931,7 +1995,7 @@ static void widen_cap(const GpPointF *endpoint, const GpPointF *nextpoint,
     {
         REAL segment_dy = nextpoint->Y-endpoint->Y;
         REAL segment_dx = nextpoint->X-endpoint->X;
-        REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+        REAL segment_length = hypotf(segment_dy, segment_dx);
         REAL distance = pen_width / 2.0;
         REAL dx, dy, dx2, dy2;
         const REAL control_point_distance = 0.5522847498307935; /* 4/3 * (sqrt(2) - 1) */
@@ -1976,7 +2040,7 @@ static void widen_cap(const GpPointF *endpoint, const GpPointF *nextpoint,
     {
         REAL segment_dy = nextpoint->Y-endpoint->Y;
         REAL segment_dx = nextpoint->X-endpoint->X;
-        REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+        REAL segment_length = hypotf(segment_dy, segment_dx);
         REAL distance = pen_width / 2.0;
         REAL dx, dy;
 
@@ -2016,7 +2080,7 @@ static void add_anchor(const GpPointF *endpoint, const GpPointF *nextpoint,
     {
         REAL segment_dy = nextpoint->Y-endpoint->Y;
         REAL segment_dx = nextpoint->X-endpoint->X;
-        REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+        REAL segment_length = hypotf(segment_dy, segment_dx);
         REAL distance = pen_width / sqrtf(2.0);
         REAL par_dx, par_dy;
         REAL perp_dx, perp_dy;
@@ -2041,7 +2105,7 @@ static void add_anchor(const GpPointF *endpoint, const GpPointF *nextpoint,
     {
         REAL segment_dy = nextpoint->Y-endpoint->Y;
         REAL segment_dx = nextpoint->X-endpoint->X;
-        REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+        REAL segment_length = hypotf(segment_dy, segment_dx);
         REAL dx, dy, dx2, dy2;
         const REAL control_point_distance = 0.55228475; /* 4/3 * (sqrt(2) - 1) */
 
@@ -2093,7 +2157,7 @@ static void add_anchor(const GpPointF *endpoint, const GpPointF *nextpoint,
     {
         REAL segment_dy = nextpoint->Y-endpoint->Y;
         REAL segment_dx = nextpoint->X-endpoint->X;
-        REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+        REAL segment_length = hypotf(segment_dy, segment_dx);
         REAL par_dx, par_dy;
         REAL perp_dx, perp_dy;
 
@@ -2117,7 +2181,7 @@ static void add_anchor(const GpPointF *endpoint, const GpPointF *nextpoint,
     {
         REAL segment_dy = nextpoint->Y - endpoint->Y;
         REAL segment_dx = nextpoint->X - endpoint->X;
-        REAL segment_length = sqrtf(segment_dy * segment_dy + segment_dx * segment_dx);
+        REAL segment_length = hypotf(segment_dy, segment_dx);
         REAL par_dx = pen_width * segment_dx / segment_length;
         REAL par_dy = pen_width * segment_dy / segment_length;
         REAL perp_dx = -par_dy;
@@ -2163,7 +2227,7 @@ static void add_anchor(const GpPointF *endpoint, const GpPointF *nextpoint,
 
         if (!custom->fill)
         {
-            tmp_points = heap_alloc_zero(custom->pathdata.Count * sizeof(GpPoint));
+            tmp_points = malloc(custom->pathdata.Count * sizeof(GpPoint));
             if (!tmp_points) {
                 ERR("Out of memory\n");
                 return;
@@ -2178,6 +2242,7 @@ static void add_anchor(const GpPointF *endpoint, const GpPointF *nextpoint,
                 widen_closed_figure(tmp_points, 0, custom->pathdata.Count - 1, pen, pen_width, last_point);
             else
                 widen_open_figure(tmp_points, 0, custom->pathdata.Count - 1, pen, pen_width, custom->strokeEndCap, NULL, custom->strokeStartCap, NULL, last_point);
+            free(tmp_points);
         }
         else
         {
@@ -2281,6 +2346,7 @@ static void widen_dashed_figure(GpPath *path, int start, int end, int closed,
     int dash_index=0;
     const REAL *dash_pattern;
     REAL *dash_pattern_scaled;
+    REAL dash_pattern_scaling = max(pen->width, 1.0);
     int dash_count;
     GpPointF *tmp_points;
     REAL segment_dy;
@@ -2319,15 +2385,15 @@ static void widen_dashed_figure(GpPath *path, int start, int end, int closed,
         break;
     }
 
-    dash_pattern_scaled = heap_alloc(dash_count * sizeof(REAL));
+    dash_pattern_scaled = malloc(dash_count * sizeof(REAL));
     if (!dash_pattern_scaled) return;
 
     for (i = 0; i < dash_count; i++)
-        dash_pattern_scaled[i] = pen->width * dash_pattern[i];
+        dash_pattern_scaled[i] = dash_pattern_scaling * dash_pattern[i];
 
-    tmp_points = heap_alloc_zero((end - start + 2) * sizeof(GpPoint));
+    tmp_points = calloc(end - start + 2, sizeof(GpPoint));
     if (!tmp_points) {
-        heap_free(dash_pattern_scaled);
+        free(dash_pattern_scaled);
         return; /* FIXME */
     }
 
@@ -2348,7 +2414,7 @@ static void widen_dashed_figure(GpPath *path, int start, int end, int closed,
 
         segment_dy = path->pathdata.Points[j].Y - path->pathdata.Points[i].Y;
         segment_dx = path->pathdata.Points[j].X - path->pathdata.Points[i].X;
-        segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+        segment_length = hypotf(segment_dy, segment_dx);
         segment_pos = 0.0;
 
         while (1)
@@ -2407,8 +2473,8 @@ static void widen_dashed_figure(GpPath *path, int start, int end, int closed,
             closed ? LineCapFlat : pen->endcap, pen->customend, last_point);
     }
 
-    heap_free(dash_pattern_scaled);
-    heap_free(tmp_points);
+    free(dash_pattern_scaled);
+    free(tmp_points);
 }
 
 GpStatus WINGDIPAPI GdipWidenPath(GpPath *path, GpPen *pen, GpMatrix *matrix,
@@ -2419,7 +2485,7 @@ GpStatus WINGDIPAPI GdipWidenPath(GpPath *path, GpPen *pen, GpMatrix *matrix,
     path_list_node_t *points=NULL, *last_point=NULL;
     int i, subpath_start=0, new_length;
 
-    TRACE("(%p,%p,%p,%0.2f)\n", path, pen, matrix, flatness);
+    TRACE("(%p,%p,%s,%0.2f)\n", path, pen, debugstr_matrix(matrix), flatness);
 
     if (!path || !pen)
         return InvalidParameter;
@@ -2573,10 +2639,10 @@ GpStatus WINGDIPAPI GdipAddPathRectangle(GpPath *path, REAL x, REAL y,
 
 fail:
     /* reverting */
-    heap_free(path->pathdata.Points);
-    heap_free(path->pathdata.Types);
+    free(path->pathdata.Points);
+    free(path->pathdata.Types);
     memcpy(path, backup, sizeof(*path));
-    heap_free(backup);
+    free(backup);
 
     return retstat;
 }
@@ -2619,10 +2685,10 @@ GpStatus WINGDIPAPI GdipAddPathRectangles(GpPath *path, GDIPCONST GpRectF *rects
 
 fail:
     /* reverting */
-    heap_free(path->pathdata.Points);
-    heap_free(path->pathdata.Types);
+    free(path->pathdata.Points);
+    free(path->pathdata.Types);
     memcpy(path, backup, sizeof(*path));
-    heap_free(backup);
+    free(backup);
 
     return retstat;
 }
@@ -2641,13 +2707,13 @@ GpStatus WINGDIPAPI GdipAddPathRectanglesI(GpPath *path, GDIPCONST GpRect *rects
     if(count < 0)
         return OutOfMemory;
 
-    rectsF = heap_alloc_zero(sizeof(GpRectF)*count);
+    rectsF = malloc(sizeof(GpRectF) * count);
 
     for(i = 0;i < count;i++)
         set_rect(&rectsF[i], rects[i].X, rects[i].Y, rects[i].Width, rects[i].Height);
 
     retstat = GdipAddPathRectangles(path, rectsF, count);
-    heap_free(rectsF);
+    free(rectsF);
 
     return retstat;
 }
