@@ -22,6 +22,8 @@
 #include <stdarg.h>
 #include <string.h>
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "wine/winuser16.h"
 #include "wownt32.h"
 #include "winerror.h"
@@ -78,6 +80,12 @@ static LRESULT defdlg_proc_callback( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp,
     return *result;
 }
 
+static LRESULT defwnd_proc_callback( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp,
+                                     LRESULT *result, void *arg )
+{
+    return *result = DefWindowProcA( hwnd, msg, wp, lp );
+}
+
 static LRESULT call_window_proc_callback( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp,
                                           LRESULT *result, void *arg )
 {
@@ -152,8 +160,10 @@ static WNDPROC16 alloc_win16_thunk( WNDPROC handle )
     {
         assert( MAX_WINPROCS16 * sizeof(WINPROC_THUNK) <= 0x10000 );
 
-        if (!(thunk_selector = GlobalAlloc16( GMEM_FIXED, MAX_WINPROCS16 * sizeof(WINPROC_THUNK) )))
+        if (!(thunk_selector = GlobalAlloc16( GMEM_FIXED | GMEM_ZEROINIT,
+                                              MAX_WINPROCS16 * sizeof(WINPROC_THUNK) )))
             return NULL;
+        FarSetOwner16( thunk_selector, 0 );
         PrestoChangoSelector16( thunk_selector, thunk_selector );
         thunk_array = GlobalLock16( thunk_selector );
         relay = GetProcAddress16( GetModuleHandle16("user"), "__wine_call_wndproc" );
@@ -239,11 +249,11 @@ static LRESULT call_window_proc16( HWND16 hwnd, UINT16 msg, WPARAM16 wParam, LPA
     /* Window procedures want ax = hInstance, ds = es = ss */
 
     memset(&context, 0, sizeof(context));
-    context.SegDs = context.SegEs = SELECTOROF(NtCurrentTeb()->WOW32Reserved);
-    if (!(context.Eax = GetWindowWord( HWND_32(hwnd), GWLP_HINSTANCE ))) context.Eax = context.SegDs;
+    context.SegDs = context.SegEs = CURRENT_SS;
+    if (!(context.Eax = (WORD)GetWindowLongA( HWND_32(hwnd), GWLP_HINSTANCE ))) context.Eax = context.SegDs;
     context.SegCs = SELECTOROF(func);
     context.Eip   = OFFSETOF(func);
-    context.Ebp   = OFFSETOF(NtCurrentTeb()->WOW32Reserved) + FIELD_OFFSET(STACK16FRAME, bp);
+    context.Ebp   = CURRENT_SP + FIELD_OFFSET(STACK16FRAME, bp);
 
     if (lParam)
     {
@@ -266,7 +276,7 @@ static LRESULT call_window_proc16( HWND16 hwnd, UINT16 msg, WPARAM16 wParam, LPA
         if (size)
         {
             memcpy( &args.u, MapSL(lParam), size );
-            lParam = PtrToUlong(NtCurrentTeb()->WOW32Reserved) - size;
+            lParam = MAKESEGPTR( CURRENT_SS, CURRENT_SP - size );
         }
     }
 
@@ -603,6 +613,7 @@ LRESULT WINPROC_CallProc16To32A( winproc_callback_t callback, HWND16 hwnd, UINT1
             CREATESTRUCT16 *cs16 = MapSL(lParam);
             CREATESTRUCTA cs;
             MDICREATESTRUCTA mdi_cs;
+            SEGPTR mdi_cs_segptr = 0;
 
             CREATESTRUCT16to32A( cs16, &cs );
             if (GetWindowLongW(hwnd32, GWL_EXSTYLE) & WS_EX_MDICHILD)
@@ -610,9 +621,15 @@ LRESULT WINPROC_CallProc16To32A( winproc_callback_t callback, HWND16 hwnd, UINT1
                 MDICREATESTRUCT16 *mdi_cs16 = MapSL(cs16->lpCreateParams);
                 MDICREATESTRUCT16to32A(mdi_cs16, &mdi_cs);
                 cs.lpCreateParams = &mdi_cs;
+                mdi_cs_segptr = cs16->lpCreateParams;
             }
             ret = callback( hwnd32, msg, wParam, (LPARAM)&cs, result, arg );
             CREATESTRUCT32Ato16( &cs, cs16 );
+            if (mdi_cs_segptr)
+            {
+                MDICREATESTRUCT32Ato16( &mdi_cs, MapSL( mdi_cs_segptr ) );
+                cs16->lpCreateParams = mdi_cs_segptr;
+            }
         }
         break;
     case WM_MDICREATE:
@@ -880,7 +897,7 @@ LRESULT WINPROC_CallProc16To32A( winproc_callback_t callback, HWND16 hwnd, UINT1
             case 1:
                 break; /* atom, nothing to do */
             case 3:
-                WARN("DDE_ACK: %lx both atom and handle... choosing handle\n", hi);
+                WARN("DDE_ACK: %Ix both atom and handle... choosing handle\n", hi);
                 /* fall through */
             case 2:
                 hi = convert_handle_16_to_32(hi, GMEM_DDESHARE);
@@ -898,6 +915,9 @@ LRESULT WINPROC_CallProc16To32A( winproc_callback_t callback, HWND16 hwnd, UINT1
     case WM_SIZECLIPBOARD:
         FIXME_(msg)( "message %04x needs translation\n", msg );
         break;
+    case WM_ERASEBKGND:
+        ret = callback( hwnd32, msg, (WPARAM)HDC_32(wParam), lParam, result, arg );
+        break;
     default:
         ret = callback( hwnd32, msg, wParam, lParam, result, arg );
         break;
@@ -927,15 +947,15 @@ LRESULT WINPROC_CallProc32ATo16( winproc_callback16_t callback, HWND hwnd, UINT
             BOOL mdi_child = (GetWindowLongW(hwnd, GWL_EXSTYLE) & WS_EX_MDICHILD);
 
             CREATESTRUCT32Ato16( cs32, &cs );
-            cs.lpszName  = MapLS( cs32->lpszName );
-            cs.lpszClass = MapLS( cs32->lpszClass );
+            cs.lpszName  = MapLS( (void *)cs32->lpszName );
+            cs.lpszClass = MapLS( (void *)cs32->lpszClass );
 
             if (mdi_child)
             {
                 MDICREATESTRUCTA *mdi_cs = cs32->lpCreateParams;
                 MDICREATESTRUCT32Ato16( mdi_cs, &mdi_cs16 );
-                mdi_cs16.szTitle = MapLS( mdi_cs->szTitle );
-                mdi_cs16.szClass = MapLS( mdi_cs->szClass );
+                mdi_cs16.szTitle = MapLS( (void *)mdi_cs->szTitle );
+                mdi_cs16.szClass = MapLS( (void *)mdi_cs->szClass );
                 cs.lpCreateParams = MapLS( &mdi_cs16 );
             }
             lParam = MapLS( &cs );
@@ -957,8 +977,8 @@ LRESULT WINPROC_CallProc32ATo16( winproc_callback16_t callback, HWND hwnd, UINT
             MDICREATESTRUCT16 cs;
 
             MDICREATESTRUCT32Ato16( cs32, &cs );
-            cs.szTitle = MapLS( cs32->szTitle );
-            cs.szClass = MapLS( cs32->szClass );
+            cs.szTitle = MapLS( (void *)cs32->szTitle );
+            cs.szClass = MapLS( (void *)cs32->szClass );
             lParam = MapLS( &cs );
             ret = callback( HWND_16(hwnd), msg, wParam, lParam, result, arg );
             UnMapLS( lParam );
@@ -1260,7 +1280,7 @@ LRESULT WINPROC_CallProc32ATo16( winproc_callback16_t callback, HWND hwnd, UINT
             case 1:
                 break; /* atom, nothing to do */
             case 3:
-                WARN("DDE_ACK: %lx both atom and handle... choosing handle\n", hi);
+                WARN("DDE_ACK: %Ix both atom and handle... choosing handle\n", hi);
                 /* fall through */
             case 2:
                 hi = convert_handle_32_to_16(hi, GMEM_DDESHARE);
@@ -1274,6 +1294,11 @@ LRESULT WINPROC_CallProc32ATo16( winproc_callback16_t callback, HWND hwnd, UINT
         lParam = MAKELPARAM( 0, convert_handle_32_to_16( lParam, GMEM_DDESHARE ));
         ret = callback( HWND_16(hwnd), msg, wParam, lParam, result, arg );
         break; /* FIXME don't know how to free allocated memory (handle) !! */
+    case WM_TIMER:
+        if (wParam & SYSTEM_TIMER_FLAG)
+            msg = WM_SYSTIMER;
+        ret = callback( HWND_16(hwnd), msg, wParam, lParam, result, arg );
+        break;
     case SBM_SETRANGE:
         ret = callback( HWND_16(hwnd), SBM_SETRANGE16, 0, MAKELPARAM(wParam, lParam), result, arg );
         break;
@@ -1480,9 +1505,9 @@ LRESULT WINAPI SendMessage16( HWND16 hwnd16, UINT16 msg, WPARAM16 wparam, LPARAM
 
         if (!(winproc = (WNDPROC16)GetWindowLong16( hwnd16, GWLP_WNDPROC ))) return 0;
 
-        TRACE_(message)("(0x%04x) [%04x] wp=%04x lp=%08lx\n", hwnd16, msg, wparam, lparam );
+        TRACE_(message)("(0x%04x) [%04x] wp=%04x lp=%08Ix\n", hwnd16, msg, wparam, lparam );
         result = CallWindowProc16( winproc, hwnd16, msg, wparam, lparam );
-        TRACE_(message)("(0x%04x) [%04x] wp=%04x lp=%08lx returned %08lx\n",
+        TRACE_(message)("(0x%04x) [%04x] wp=%04x lp=%08Ix returned %08Ix\n",
                         hwnd16, msg, wparam, lparam, result );
     }
     else  /* map to 32-bit unicode for inter-thread/process message */
@@ -1600,78 +1625,8 @@ BOOL16 WINAPI PeekMessage32_16( MSG32_16 *msg16, HWND16 hwnd16,
 LRESULT WINAPI DefWindowProc16( HWND16 hwnd16, UINT16 msg, WPARAM16 wParam, LPARAM lParam )
 {
     LRESULT result;
-    HWND hwnd = WIN_Handle32( hwnd16 );
-
-    switch(msg)
-    {
-    case WM_NCCREATE:
-        {
-            CREATESTRUCT16 *cs16 = MapSL(lParam);
-            CREATESTRUCTA cs32;
-
-            cs32.lpCreateParams = ULongToPtr(cs16->lpCreateParams);
-            cs32.hInstance      = HINSTANCE_32(cs16->hInstance);
-            cs32.hMenu          = HMENU_32(cs16->hMenu);
-            cs32.hwndParent     = WIN_Handle32(cs16->hwndParent);
-            cs32.cy             = cs16->cy;
-            cs32.cx             = cs16->cx;
-            cs32.y              = cs16->y;
-            cs32.x              = cs16->x;
-            cs32.style          = cs16->style;
-            cs32.dwExStyle      = cs16->dwExStyle;
-            cs32.lpszName       = MapSL(cs16->lpszName);
-            cs32.lpszClass      = MapSL(cs16->lpszClass);
-            return DefWindowProcA( hwnd, msg, wParam, (LPARAM)&cs32 );
-        }
-    case WM_NCCALCSIZE:
-        {
-            RECT16 *rect16 = MapSL(lParam);
-            RECT rect32;
-
-            rect32.left    = rect16->left;
-            rect32.top     = rect16->top;
-            rect32.right   = rect16->right;
-            rect32.bottom  = rect16->bottom;
-
-            result = DefWindowProcA( hwnd, msg, wParam, (LPARAM)&rect32 );
-
-            rect16->left   = rect32.left;
-            rect16->top    = rect32.top;
-            rect16->right  = rect32.right;
-            rect16->bottom = rect32.bottom;
-            return result;
-        }
-    case WM_WINDOWPOSCHANGING:
-    case WM_WINDOWPOSCHANGED:
-        {
-            WINDOWPOS16 *pos16 = MapSL(lParam);
-            WINDOWPOS pos32;
-
-            pos32.hwnd             = WIN_Handle32(pos16->hwnd);
-            pos32.hwndInsertAfter  = WIN_Handle32(pos16->hwndInsertAfter);
-            pos32.x                = pos16->x;
-            pos32.y                = pos16->y;
-            pos32.cx               = pos16->cx;
-            pos32.cy               = pos16->cy;
-            pos32.flags            = pos16->flags;
-
-            result = DefWindowProcA( hwnd, msg, wParam, (LPARAM)&pos32 );
-
-            pos16->hwnd            = HWND_16(pos32.hwnd);
-            pos16->hwndInsertAfter = HWND_16(pos32.hwndInsertAfter);
-            pos16->x               = pos32.x;
-            pos16->y               = pos32.y;
-            pos16->cx              = pos32.cx;
-            pos16->cy              = pos32.cy;
-            pos16->flags           = pos32.flags;
-            return result;
-        }
-    case WM_GETTEXT:
-    case WM_SETTEXT:
-        return DefWindowProcA( hwnd, msg, wParam, (LPARAM)MapSL(lParam) );
-    default:
-        return DefWindowProcA( hwnd, msg, wParam, lParam );
-    }
+    WINPROC_CallProc16To32A( defwnd_proc_callback, hwnd16, msg, wParam, lParam, &result, 0 );
+    return result;
 }
 
 
@@ -1777,9 +1732,9 @@ LONG WINAPI DispatchMessage16( const MSG16* msg )
         SetLastError( ERROR_INVALID_WINDOW_HANDLE );
         return 0;
     }
-    TRACE_(message)("(0x%04x) [%04x] wp=%04x lp=%08lx\n", msg->hwnd, msg->message, msg->wParam, msg->lParam );
+    TRACE_(message)("(0x%04x) [%04x] wp=%04x lp=%08Ix\n", msg->hwnd, msg->message, msg->wParam, msg->lParam );
     retval = CallWindowProc16( winproc, msg->hwnd, msg->message, msg->wParam, msg->lParam );
-    TRACE_(message)("(0x%04x) [%04x] wp=%04x lp=%08lx returned %08lx\n",
+    TRACE_(message)("(0x%04x) [%04x] wp=%04x lp=%08Ix returned %08Ix\n",
                     msg->hwnd, msg->message, msg->wParam, msg->lParam, retval );
     return retval;
 }
@@ -2091,7 +2046,6 @@ static LRESULT combo_proc16( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam,
 
 static void edit_lock_buffer( HWND hwnd )
 {
-    STACK16FRAME* stack16 = MapSL(PtrToUlong(NtCurrentTeb()->WOW32Reserved));
     HLOCAL16 hloc16 = GetWindowWord( hwnd, GWW_HANDLE16 );
     HANDLE16 oldDS;
     HLOCAL hloc32;
@@ -2100,8 +2054,8 @@ static void edit_lock_buffer( HWND hwnd )
     if (!hloc16) return;
     if (!(hloc32 = (HLOCAL)wow_handlers32.edit_proc( hwnd, EM_GETHANDLE, 0, 0, FALSE ))) return;
 
-    oldDS = stack16->ds;
-    stack16->ds = GetWindowLongPtrW( hwnd, GWLP_HINSTANCE );
+    oldDS = CURRENT_DS;
+    CURRENT_DS = GetWindowLongPtrW( hwnd, GWLP_HINSTANCE );
     size = LocalSize16(hloc16);
     if (LocalReAlloc( hloc32, size, LMEM_MOVEABLE ))
     {
@@ -2111,13 +2065,11 @@ static void edit_lock_buffer( HWND hwnd )
         LocalUnlock( hloc32 );
         LocalUnlock16( hloc16 );
     }
-    stack16->ds = oldDS;
-
+    CURRENT_DS = oldDS;
 }
 
 static void edit_unlock_buffer( HWND hwnd )
 {
-    STACK16FRAME* stack16 = MapSL(PtrToUlong(NtCurrentTeb()->WOW32Reserved));
     HLOCAL16 hloc16 = GetWindowWord( hwnd, GWW_HANDLE16 );
     HANDLE16 oldDS;
     HLOCAL hloc32;
@@ -2127,8 +2079,8 @@ static void edit_unlock_buffer( HWND hwnd )
     if (!(hloc32 = (HLOCAL)wow_handlers32.edit_proc( hwnd, EM_GETHANDLE, 0, 0, FALSE ))) return;
     size = LocalSize( hloc32 );
 
-    oldDS = stack16->ds;
-    stack16->ds = GetWindowLongPtrW( hwnd, GWLP_HINSTANCE );
+    oldDS = CURRENT_DS;
+    CURRENT_DS = GetWindowLongPtrW( hwnd, GWLP_HINSTANCE );
     if (LocalReAlloc16( hloc16, size, LMEM_MOVEABLE ))
     {
         char *text = LocalLock( hloc32 );
@@ -2137,7 +2089,7 @@ static void edit_unlock_buffer( HWND hwnd )
         LocalUnlock( hloc32 );
         LocalUnlock16( hloc16 );
     }
-    stack16->ds = oldDS;
+    CURRENT_DS = oldDS;
 }
 
 static HLOCAL16 edit_get_handle( HWND hwnd )
@@ -2145,7 +2097,6 @@ static HLOCAL16 edit_get_handle( HWND hwnd )
     CHAR *textA;
     UINT alloc_size;
     HLOCAL hloc;
-    STACK16FRAME* stack16;
     HANDLE16 oldDS;
     HLOCAL16 hloc16 = GetWindowWord( hwnd, GWW_HANDLE16 );
 
@@ -2154,13 +2105,12 @@ static HLOCAL16 edit_get_handle( HWND hwnd )
     if (!(hloc = (HLOCAL)wow_handlers32.edit_proc( hwnd, EM_GETHANDLE, 0, 0, FALSE ))) return 0;
     alloc_size = LocalSize( hloc );
 
-    stack16 = MapSL(PtrToUlong(NtCurrentTeb()->WOW32Reserved));
-    oldDS = stack16->ds;
-    stack16->ds = GetWindowLongPtrW( hwnd, GWLP_HINSTANCE );
+    oldDS = CURRENT_DS;
+    CURRENT_DS = GetWindowLongPtrW( hwnd, GWLP_HINSTANCE );
 
     if (!LocalHeapSize16())
     {
-        if (!LocalInit16(stack16->ds, 0, GlobalSize16(stack16->ds)))
+        if (!LocalInit16(CURRENT_DS, 0, GlobalSize16(CURRENT_DS)))
         {
             ERR("could not initialize local heap\n");
             goto done;
@@ -2186,15 +2136,14 @@ static HLOCAL16 edit_get_handle( HWND hwnd )
     SetWindowWord( hwnd, GWW_HANDLE16, hloc16 );
 
 done:
-    stack16->ds = oldDS;
+    CURRENT_DS = oldDS;
     return hloc16;
 }
 
 static void edit_set_handle( HWND hwnd, HLOCAL16 hloc16 )
 {
-    STACK16FRAME* stack16 = MapSL(PtrToUlong(NtCurrentTeb()->WOW32Reserved));
     HINSTANCE16 hInstance = GetWindowLongPtrW( hwnd, GWLP_HINSTANCE );
-    HANDLE16 oldDS = stack16->ds;
+    HANDLE16 oldDS = CURRENT_DS;
     HLOCAL hloc32;
     INT count;
     CHAR *text;
@@ -2202,7 +2151,7 @@ static void edit_set_handle( HWND hwnd, HLOCAL16 hloc16 )
     if (!(GetWindowLongW( hwnd, GWL_STYLE ) & ES_MULTILINE)) return;
     if (!hloc16) return;
 
-    stack16->ds = hInstance;
+    CURRENT_DS = hInstance;
     count = LocalSize16(hloc16);
     text = MapSL(LocalLock16(hloc16));
     if ((hloc32 = LocalAlloc(LMEM_MOVEABLE, count)))
@@ -2212,8 +2161,7 @@ static void edit_set_handle( HWND hwnd, HLOCAL16 hloc16 )
         LocalUnlock16(hloc16);
         SetWindowWord( hwnd, GWW_HANDLE16, hloc16 );
     }
-    stack16->ds = oldDS;
-
+    CURRENT_DS = oldDS;
     if (hloc32) wow_handlers32.edit_proc( hwnd, EM_SETHANDLE, (WPARAM)hloc32, 0, FALSE );
 }
 
@@ -2222,13 +2170,12 @@ static void edit_destroy_handle( HWND hwnd )
     HLOCAL16 hloc16 = GetWindowWord( hwnd, GWW_HANDLE16 );
     if (hloc16)
     {
-        STACK16FRAME* stack16 = MapSL(PtrToUlong(NtCurrentTeb()->WOW32Reserved));
-        HANDLE16 oldDS = stack16->ds;
+        HANDLE16 oldDS = CURRENT_DS;
 
-        stack16->ds = GetWindowLongPtrW( hwnd, GWLP_HINSTANCE );
+        CURRENT_DS = GetWindowLongPtrW( hwnd, GWLP_HINSTANCE );
         while (LocalUnlock16(hloc16)) ;
         LocalFree16(hloc16);
-        stack16->ds = oldDS;
+        CURRENT_DS = oldDS;
         SetWindowWord( hwnd, GWW_HANDLE16, 0 );
     }
 }
@@ -2623,20 +2570,6 @@ static LRESULT static_proc16( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam,
 }
 
 
-/***********************************************************************
- *           wait_message16
- */
-static DWORD wait_message16( DWORD count, const HANDLE *handles, DWORD timeout, DWORD mask, DWORD flags )
-{
-    DWORD lock, ret;
-
-    ReleaseThunkLock( &lock );
-    ret = wow_handlers32.wait_message( count, handles, timeout, mask, flags );
-    RestoreThunkLock( lock );
-    return ret;
-}
-
-
 /***********************************************************************
  *           create_window16
  */
@@ -2650,17 +2583,31 @@ HWND create_window16( CREATESTRUCTW *cs, LPCWSTR className, HINSTANCE instance,
 }
 
 
-/***********************************************************************
- *           free_icon_param
- */
-static void free_icon_param( ULONG_PTR param )
+static NTSTATUS WINAPI User16CallFreeIcon( void *args, ULONG size )
 {
-    GlobalFree16( LOWORD(param) );
+    ULONG *param = args;
+    GlobalFree16( LOWORD(*param) );
+    return STATUS_SUCCESS;
+}
+
+
+static NTSTATUS WINAPI User16ThunkLock( void *args, ULONG size )
+{
+    DWORD *param = args;
+    if (size != sizeof(DWORD))
+    {
+        DWORD lock;
+        ReleaseThunkLock( &lock );
+        return NtCallbackReturn( &lock, sizeof(lock), STATUS_SUCCESS );
+    }
+    RestoreThunkLock( *param );
+    return STATUS_SUCCESS;
 }
 
 
 void register_wow_handlers(void)
 {
+    KERNEL_CALLBACK_PROC *callback_table = NtCurrentTeb()->Peb->KernelCallbackTable;
     static const struct wow_handlers16 handlers16 =
     {
         button_proc16,
@@ -2670,12 +2617,15 @@ void register_wow_handlers(void)
         mdiclient_proc16,
         scrollbar_proc16,
         static_proc16,
-        wait_message16,
         create_window16,
         call_window_proc_Ato16,
         call_dialog_proc_Ato16,
-        free_icon_param
     };
 
+    callback_table[NtUserCallFreeIcon] = User16CallFreeIcon;
+    callback_table[NtUserThunkLock]    = User16ThunkLock;
+
+    NtUserEnableThunkLock( TRUE );
+
     UserRegisterWowHandlers( &handlers16, &wow_handlers32 );
 }
