@@ -18,43 +18,16 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include <assert.h>
-#include <limits.h>
-#include <stdarg.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <wchar.h>
-
-#include "windef.h"
-#include "winbase.h"
-#include "winnls.h"
-#include "wingdi.h"
-#include "winuser.h"
-#include "winreg.h"
-#include "wine/wingdi16.h"
-#include "winerror.h"
-
-#include "initguid.h"
-#include "d3dkmdt.h"
-#include "devguid.h"
-#include "setupapi.h"
-#include "controls.h"
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "user_private.h"
+#include "controls.h"
 #include "wine/asm.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(system);
 
 
-DEFINE_DEVPROPKEY(DEVPROPKEY_MONITOR_GPU_LUID, 0xca085853, 0x16ce, 0x48aa, 0xb1, 0x14, 0xde, 0x9c, 0x72, 0x33, 0x42, 0x23, 1);
-DEFINE_DEVPROPKEY(DEVPROPKEY_MONITOR_OUTPUT_ID, 0xca085853, 0x16ce, 0x48aa, 0xb1, 0x14, 0xde, 0x9c, 0x72, 0x33, 0x42, 0x23, 2);
-
-/* Wine specific monitor properties */
-DEFINE_DEVPROPKEY(WINE_DEVPROPKEY_MONITOR_STATEFLAGS, 0x233a9ef3, 0xafc4, 0x4abd, 0xb5, 0x64, 0xc3, 0x2f, 0x21, 0xf1, 0x53, 0x5b, 2);
-DEFINE_DEVPROPKEY(WINE_DEVPROPKEY_MONITOR_ADAPTERNAME, 0x233a9ef3, 0xafc4, 0x4abd, 0xb5, 0x64, 0xc3, 0x2f, 0x21, 0xf1, 0x53, 0x5b, 5);
-
-
 static HDC display_dc;
 static CRITICAL_SECTION display_dc_section;
 static CRITICAL_SECTION_DEBUG critsect_debug =
@@ -160,43 +133,6 @@ static void SYSPARAMS_NonClientMetrics32ATo32W( const NONCLIENTMETRICSA* lpnm32A
 
 /* Helper functions to retrieve monitors info */
 
-static BOOL CALLBACK get_virtual_screen_proc( HMONITOR monitor, HDC hdc, LPRECT rect, LPARAM lp )
-{
-    RECT *virtual_rect = (RECT *)lp;
-
-    UnionRect( virtual_rect, virtual_rect, rect );
-    return TRUE;
-}
-
-RECT get_virtual_screen_rect(void)
-{
-    RECT rect = {0};
-
-    NtUserEnumDisplayMonitors( 0, NULL, get_virtual_screen_proc, (LPARAM)&rect );
-    return rect;
-}
-
-static BOOL CALLBACK get_primary_monitor_proc( HMONITOR monitor, HDC hdc, LPRECT rect, LPARAM lp )
-{
-    RECT *primary_rect = (RECT *)lp;
-
-    if (!rect->top && !rect->left && rect->right && rect->bottom)
-    {
-        *primary_rect = *rect;
-        return FALSE;
-    }
-
-    return TRUE;
-}
-
-RECT get_primary_monitor_rect(void)
-{
-    RECT rect = {0};
-
-    NtUserEnumDisplayMonitors( 0, NULL, get_primary_monitor_proc, (LPARAM)&rect );
-    return rect;
-}
-
 HDC get_display_dc(void)
 {
     EnterCriticalSection( &display_dc_section );
@@ -220,32 +156,6 @@ void release_display_dc( HDC hdc )
     LeaveCriticalSection( &display_dc_section );
 }
 
-static HANDLE get_display_device_init_mutex( void )
-{
-    HANDLE mutex = CreateMutexW( NULL, FALSE, L"display_device_init" );
-
-    WaitForSingleObject( mutex, INFINITE );
-    return mutex;
-}
-
-static void release_display_device_init_mutex( HANDLE mutex )
-{
-    ReleaseMutex( mutex );
-    CloseHandle( mutex );
-}
-
-/* Wait until graphics driver is loaded by explorer */
-void wait_graphics_driver_ready(void)
-{
-    static BOOL ready = FALSE;
-
-    if (!ready)
-    {
-        SendMessageW( GetDesktopWindow(), WM_NULL, 0, 0 );
-        ready = TRUE;
-    }
-}
-
 /***********************************************************************
  *           SYSPARAMS_Init
  */
@@ -838,6 +748,24 @@ DPI_AWARENESS_CONTEXT WINAPI SetThreadDpiAwarenessContext( DPI_AWARENESS_CONTEXT
     return ULongToHandle( prev );
 }
 
+/**********************************************************************
+ *              GetThreadDpiHostingBehavior   (USER32.@)
+ */
+DPI_HOSTING_BEHAVIOR WINAPI GetThreadDpiHostingBehavior( void )
+{
+    FIXME("(): stub\n");
+    return DPI_HOSTING_BEHAVIOR_DEFAULT;
+}
+
+/**********************************************************************
+ *              SetThreadDpiHostingBehavior   (USER32.@)
+ */
+DPI_HOSTING_BEHAVIOR WINAPI SetThreadDpiHostingBehavior( DPI_HOSTING_BEHAVIOR value )
+{
+    FIXME("(%d): stub\n", value);
+    return DPI_HOSTING_BEHAVIOR_DEFAULT;
+}
+
 /***********************************************************************
  *		MonitorFromRect (USER32.@)
  */
@@ -923,14 +851,17 @@ __ASM_GLOBAL_FUNC( enum_mon_callback_wrapper,
     "ret" )
 #endif /* __i386__ */
 
-BOOL WINAPI User32CallEnumDisplayMonitor( struct enum_display_monitor_params *params, ULONG size )
+NTSTATUS WINAPI User32CallEnumDisplayMonitor( void *args, ULONG size )
 {
+    struct enum_display_monitor_params *params = args;
+    BOOL ret;
 #ifdef __i386__
-    return enum_mon_callback_wrapper( params->proc, params->monitor, params->hdc,
-                                      &params->rect, params->lparam );
+    ret = enum_mon_callback_wrapper( params->proc, params->monitor, params->hdc,
+                                     &params->rect, params->lparam );
 #else
-    return params->proc( params->monitor, params->hdc, &params->rect, params->lparam );
+    ret = params->proc( params->monitor, params->hdc, &params->rect, params->lparam );
 #endif
+    return NtCallbackReturn( &ret, sizeof(ret), STATUS_SUCCESS );
 }
 
 /***********************************************************************
@@ -994,7 +925,7 @@ BOOL WINAPI GetAutoRotationState( AR_STATE *state )
 }
 
 /**********************************************************************
- *              GetDisplayAutoRotationPreferences [USER32.@]
+ *              GetDisplayAutoRotationPreferences (USER32.@)
  */
 BOOL WINAPI GetDisplayAutoRotationPreferences( ORIENTATION_PREFERENCE *orientation )
 {
@@ -1003,6 +934,15 @@ BOOL WINAPI GetDisplayAutoRotationPreferences( ORIENTATION_PREFERENCE *orientati
     return TRUE;
 }
 
+/**********************************************************************
+ *              SetDisplayAutoRotationPreferences (USER32.@)
+ */
+BOOL WINAPI SetDisplayAutoRotationPreferences( ORIENTATION_PREFERENCE orientation )
+{
+    FIXME("(%d): stub\n", orientation);
+    return TRUE;
+}
+
 /* physical<->logical mapping functions from win8 that are nops in later versions */
 
 /***********************************************************************
@@ -1045,367 +985,12 @@ BOOL WINAPI PhysicalToLogicalPoint( HWND hwnd, POINT *point )
     return TRUE;
 }
 
-static DISPLAYCONFIG_ROTATION get_dc_rotation(const DEVMODEW *devmode)
-{
-    if (devmode->dmFields & DM_DISPLAYORIENTATION)
-        return devmode->dmDisplayOrientation + 1;
-    else
-        return DISPLAYCONFIG_ROTATION_IDENTITY;
-}
-
-static DISPLAYCONFIG_SCANLINE_ORDERING get_dc_scanline_ordering(const DEVMODEW *devmode)
-{
-    if (!(devmode->dmFields & DM_DISPLAYFLAGS))
-        return DISPLAYCONFIG_SCANLINE_ORDERING_UNSPECIFIED;
-    else if (devmode->dmDisplayFlags & DM_INTERLACED)
-        return DISPLAYCONFIG_SCANLINE_ORDERING_INTERLACED;
-    else
-        return DISPLAYCONFIG_SCANLINE_ORDERING_PROGRESSIVE;
-}
-
-static DISPLAYCONFIG_PIXELFORMAT get_dc_pixelformat(DWORD dmBitsPerPel)
-{
-    if ((dmBitsPerPel == 8) || (dmBitsPerPel == 16) ||
-        (dmBitsPerPel == 24) || (dmBitsPerPel == 32))
-        return dmBitsPerPel / 8;
-    else
-        return DISPLAYCONFIG_PIXELFORMAT_NONGDI;
-}
-
-static void set_mode_target_info(DISPLAYCONFIG_MODE_INFO *info, const LUID *gpu_luid, UINT32 target_id,
-                                 UINT32 flags, const DEVMODEW *devmode)
-{
-    DISPLAYCONFIG_TARGET_MODE *mode = &info->targetMode;
-
-    info->infoType = DISPLAYCONFIG_MODE_INFO_TYPE_TARGET;
-    info->adapterId = *gpu_luid;
-    info->id = target_id;
-
-    /* FIXME: Populate pixelRate/hSyncFreq/totalSize with real data */
-    mode->targetVideoSignalInfo.pixelRate = devmode->dmDisplayFrequency * devmode->dmPelsWidth * devmode->dmPelsHeight;
-    mode->targetVideoSignalInfo.hSyncFreq.Numerator = devmode->dmDisplayFrequency * devmode->dmPelsWidth;
-    mode->targetVideoSignalInfo.hSyncFreq.Denominator = 1;
-    mode->targetVideoSignalInfo.vSyncFreq.Numerator = devmode->dmDisplayFrequency;
-    mode->targetVideoSignalInfo.vSyncFreq.Denominator = 1;
-    mode->targetVideoSignalInfo.activeSize.cx = devmode->dmPelsWidth;
-    mode->targetVideoSignalInfo.activeSize.cy = devmode->dmPelsHeight;
-    if (flags & QDC_DATABASE_CURRENT)
-    {
-        mode->targetVideoSignalInfo.totalSize.cx = 0;
-        mode->targetVideoSignalInfo.totalSize.cy = 0;
-    }
-    else
-    {
-        mode->targetVideoSignalInfo.totalSize.cx = devmode->dmPelsWidth;
-        mode->targetVideoSignalInfo.totalSize.cy = devmode->dmPelsHeight;
-    }
-    mode->targetVideoSignalInfo.videoStandard = D3DKMDT_VSS_OTHER;
-    mode->targetVideoSignalInfo.scanLineOrdering = get_dc_scanline_ordering(devmode);
-}
-
-static void set_path_target_info(DISPLAYCONFIG_PATH_TARGET_INFO *info, const LUID *gpu_luid,
-                                 UINT32 target_id, UINT32 mode_index, const DEVMODEW *devmode)
-{
-    info->adapterId = *gpu_luid;
-    info->id = target_id;
-    info->modeInfoIdx = mode_index;
-    info->outputTechnology = DISPLAYCONFIG_OUTPUT_TECHNOLOGY_DISPLAYPORT_EXTERNAL;
-    info->rotation = get_dc_rotation(devmode);
-    info->scaling = DISPLAYCONFIG_SCALING_IDENTITY;
-    info->refreshRate.Numerator = devmode->dmDisplayFrequency;
-    info->refreshRate.Denominator = 1;
-    info->scanLineOrdering = get_dc_scanline_ordering(devmode);
-    info->targetAvailable = TRUE;
-    info->statusFlags = DISPLAYCONFIG_TARGET_IN_USE;
-}
-
-static void set_mode_source_info(DISPLAYCONFIG_MODE_INFO *info, const LUID *gpu_luid,
-                                 UINT32 source_id, const DEVMODEW *devmode)
-{
-    DISPLAYCONFIG_SOURCE_MODE *mode = &(info->sourceMode);
-
-    info->infoType = DISPLAYCONFIG_MODE_INFO_TYPE_SOURCE;
-    info->adapterId = *gpu_luid;
-    info->id = source_id;
-
-    mode->width = devmode->dmPelsWidth;
-    mode->height = devmode->dmPelsHeight;
-    mode->pixelFormat = get_dc_pixelformat(devmode->dmBitsPerPel);
-    if (devmode->dmFields & DM_POSITION)
-    {
-        mode->position = devmode->dmPosition;
-    }
-    else
-    {
-        mode->position.x = 0;
-        mode->position.y = 0;
-    }
-}
-
-static void set_path_source_info(DISPLAYCONFIG_PATH_SOURCE_INFO *info, const LUID *gpu_luid,
-                                 UINT32 source_id, UINT32 mode_index)
-{
-    info->adapterId = *gpu_luid;
-    info->id = source_id;
-    info->modeInfoIdx = mode_index;
-    info->statusFlags = DISPLAYCONFIG_SOURCE_IN_USE;
-}
-
-static BOOL source_mode_exists(const DISPLAYCONFIG_MODE_INFO *modeinfo, UINT32 num_modes,
-                               UINT32 source_id, UINT32 *found_mode_index)
-{
-    UINT32 i;
-
-    for (i = 0; i < num_modes; i++)
-    {
-        if (modeinfo[i].infoType == DISPLAYCONFIG_MODE_INFO_TYPE_SOURCE &&
-            modeinfo[i].id == source_id)
-        {
-            *found_mode_index = i;
-            return TRUE;
-        }
-    }
-    return FALSE;
-}
-
-/***********************************************************************
- *              QueryDisplayConfig (USER32.@)
- */
-LONG WINAPI QueryDisplayConfig(UINT32 flags, UINT32 *numpathelements, DISPLAYCONFIG_PATH_INFO *pathinfo,
-                               UINT32 *numinfoelements, DISPLAYCONFIG_MODE_INFO *modeinfo,
-                               DISPLAYCONFIG_TOPOLOGY_ID *topologyid)
-{
-    LONG adapter_index, ret;
-    HANDLE mutex;
-    HDEVINFO devinfo;
-    SP_DEVINFO_DATA device_data = {sizeof(device_data)};
-    DWORD monitor_index = 0, state_flags, type;
-    UINT32 output_id, source_mode_index, path_index = 0, mode_index = 0;
-    LUID gpu_luid;
-    WCHAR device_name[CCHDEVICENAME];
-    DEVMODEW devmode;
-
-    FIXME("(%08x %p %p %p %p %p): semi-stub\n", flags, numpathelements, pathinfo, numinfoelements, modeinfo, topologyid);
-
-    if (!numpathelements || !numinfoelements)
-        return ERROR_INVALID_PARAMETER;
-
-    if (!*numpathelements || !*numinfoelements)
-        return ERROR_INVALID_PARAMETER;
-
-    if (flags != QDC_ALL_PATHS &&
-        flags != QDC_ONLY_ACTIVE_PATHS &&
-        flags != QDC_DATABASE_CURRENT)
-        return ERROR_INVALID_PARAMETER;
-
-    if (((flags == QDC_DATABASE_CURRENT) && !topologyid) ||
-        ((flags != QDC_DATABASE_CURRENT) && topologyid))
-        return ERROR_INVALID_PARAMETER;
-
-    if (flags != QDC_ONLY_ACTIVE_PATHS)
-        FIXME("only returning active paths\n");
-
-    if (topologyid)
-    {
-        FIXME("setting toplogyid to DISPLAYCONFIG_TOPOLOGY_INTERNAL\n");
-        *topologyid = DISPLAYCONFIG_TOPOLOGY_INTERNAL;
-    }
-
-    wait_graphics_driver_ready();
-    mutex = get_display_device_init_mutex();
-
-    /* Iterate through "targets"/monitors.
-     * Each target corresponds to a path, and each path corresponds to one or two unique modes.
-     */
-    devinfo = SetupDiGetClassDevsW(&GUID_DEVCLASS_MONITOR, L"DISPLAY", NULL, DIGCF_PRESENT);
-    if (devinfo == INVALID_HANDLE_VALUE)
-    {
-        ret = ERROR_GEN_FAILURE;
-        goto done;
-    }
-
-    ret = ERROR_GEN_FAILURE;
-    while (SetupDiEnumDeviceInfo(devinfo, monitor_index++, &device_data))
-    {
-        /* Only count active monitors */
-        if (!SetupDiGetDevicePropertyW(devinfo, &device_data, &WINE_DEVPROPKEY_MONITOR_STATEFLAGS,
-                                       &type, (BYTE *)&state_flags, sizeof(state_flags), NULL, 0))
-            goto done;
-        if (!(state_flags & DISPLAY_DEVICE_ACTIVE))
-            continue;
-
-        if (!SetupDiGetDevicePropertyW(devinfo, &device_data, &DEVPROPKEY_MONITOR_GPU_LUID,
-                                       &type, (BYTE *)&gpu_luid, sizeof(gpu_luid), NULL, 0))
-            goto done;
-
-        if (!SetupDiGetDevicePropertyW(devinfo, &device_data, &DEVPROPKEY_MONITOR_OUTPUT_ID,
-                                       &type, (BYTE *)&output_id, sizeof(output_id), NULL, 0))
-            goto done;
-
-        if (!SetupDiGetDevicePropertyW(devinfo, &device_data, &WINE_DEVPROPKEY_MONITOR_ADAPTERNAME,
-                                       &type, (BYTE *)device_name, sizeof(device_name), NULL, 0))
-            goto done;
-
-        memset(&devmode, 0, sizeof(devmode));
-        devmode.dmSize = sizeof(devmode);
-        if (!EnumDisplaySettingsW(device_name, ENUM_CURRENT_SETTINGS, &devmode))
-            goto done;
-
-        /* Extract the adapter index from device_name to use as the source ID */
-        adapter_index = wcstol(device_name + lstrlenW(L"\\\\.\\DISPLAY"), NULL, 10);
-        adapter_index--;
-
-        if (path_index == *numpathelements || mode_index == *numinfoelements)
-        {
-            ret = ERROR_INSUFFICIENT_BUFFER;
-            goto done;
-        }
-
-        pathinfo[path_index].flags = DISPLAYCONFIG_PATH_ACTIVE;
-        set_mode_target_info(&modeinfo[mode_index], &gpu_luid, output_id, flags, &devmode);
-        set_path_target_info(&(pathinfo[path_index].targetInfo), &gpu_luid, output_id, mode_index, &devmode);
-
-        mode_index++;
-        if (mode_index == *numinfoelements)
-        {
-            ret = ERROR_INSUFFICIENT_BUFFER;
-            goto done;
-        }
-
-        /* Multiple targets can be driven by the same source, ensure a mode
-         * hasn't already been added for this source.
-         */
-        if (!source_mode_exists(modeinfo, mode_index, adapter_index, &source_mode_index))
-        {
-            set_mode_source_info(&modeinfo[mode_index], &gpu_luid, adapter_index, &devmode);
-            source_mode_index = mode_index;
-            mode_index++;
-        }
-        set_path_source_info(&(pathinfo[path_index].sourceInfo), &gpu_luid, adapter_index, source_mode_index);
-        path_index++;
-    }
-
-    *numpathelements = path_index;
-    *numinfoelements = mode_index;
-    ret = ERROR_SUCCESS;
-
-done:
-    SetupDiDestroyDeviceInfoList(devinfo);
-    release_display_device_init_mutex(mutex);
-    return ret;
-}
-
 /***********************************************************************
  *              DisplayConfigGetDeviceInfo (USER32.@)
  */
 LONG WINAPI DisplayConfigGetDeviceInfo(DISPLAYCONFIG_DEVICE_INFO_HEADER *packet)
 {
-    LONG ret = ERROR_GEN_FAILURE;
-    HANDLE mutex;
-    HDEVINFO devinfo;
-    SP_DEVINFO_DATA device_data = {sizeof(device_data)};
-    DWORD index = 0, type;
-    LUID gpu_luid;
-
-    TRACE("(%p)\n", packet);
-
-    if (!packet || packet->size < sizeof(*packet))
-        return ERROR_GEN_FAILURE;
-    wait_graphics_driver_ready();
-
-    switch (packet->type)
-    {
-    case DISPLAYCONFIG_DEVICE_INFO_GET_SOURCE_NAME:
-    {
-        DISPLAYCONFIG_SOURCE_DEVICE_NAME *source_name = (DISPLAYCONFIG_SOURCE_DEVICE_NAME *)packet;
-        WCHAR device_name[CCHDEVICENAME];
-        LONG source_id;
-
-        TRACE("DISPLAYCONFIG_DEVICE_INFO_GET_SOURCE_NAME\n");
-
-        if (packet->size < sizeof(*source_name))
-            return ERROR_INVALID_PARAMETER;
-
-        mutex = get_display_device_init_mutex();
-        devinfo = SetupDiGetClassDevsW(&GUID_DEVCLASS_MONITOR, L"DISPLAY", NULL, DIGCF_PRESENT);
-        if (devinfo == INVALID_HANDLE_VALUE)
-        {
-            release_display_device_init_mutex(mutex);
-            return ret;
-        }
-
-        while (SetupDiEnumDeviceInfo(devinfo, index++, &device_data))
-        {
-            if (!SetupDiGetDevicePropertyW(devinfo, &device_data, &DEVPROPKEY_MONITOR_GPU_LUID,
-                                           &type, (BYTE *)&gpu_luid, sizeof(gpu_luid), NULL, 0))
-                continue;
-
-            if ((source_name->header.adapterId.LowPart != gpu_luid.LowPart) ||
-                (source_name->header.adapterId.HighPart != gpu_luid.HighPart))
-                continue;
-
-            /* QueryDisplayConfig() derives the source ID from the adapter name. */
-            if (!SetupDiGetDevicePropertyW(devinfo, &device_data, &WINE_DEVPROPKEY_MONITOR_ADAPTERNAME,
-                                           &type, (BYTE *)device_name, sizeof(device_name), NULL, 0))
-                continue;
-
-            source_id = wcstol(device_name + lstrlenW(L"\\\\.\\DISPLAY"), NULL, 10);
-            source_id--;
-            if (source_name->header.id != source_id)
-                continue;
-
-            lstrcpyW(source_name->viewGdiDeviceName, device_name);
-            ret = ERROR_SUCCESS;
-            break;
-        }
-        SetupDiDestroyDeviceInfoList(devinfo);
-        release_display_device_init_mutex(mutex);
-        return ret;
-    }
-    case DISPLAYCONFIG_DEVICE_INFO_GET_TARGET_NAME:
-    {
-        DISPLAYCONFIG_TARGET_DEVICE_NAME *target_name = (DISPLAYCONFIG_TARGET_DEVICE_NAME *)packet;
-
-        FIXME("DISPLAYCONFIG_DEVICE_INFO_GET_TARGET_NAME: stub\n");
-
-        if (packet->size < sizeof(*target_name))
-            return ERROR_INVALID_PARAMETER;
-
-        return ERROR_NOT_SUPPORTED;
-    }
-    case DISPLAYCONFIG_DEVICE_INFO_GET_TARGET_PREFERRED_MODE:
-    {
-        DISPLAYCONFIG_TARGET_PREFERRED_MODE *preferred_mode = (DISPLAYCONFIG_TARGET_PREFERRED_MODE *)packet;
-
-        FIXME("DISPLAYCONFIG_DEVICE_INFO_GET_TARGET_PREFERRED_MODE: stub\n");
-
-        if (packet->size < sizeof(*preferred_mode))
-            return ERROR_INVALID_PARAMETER;
-
-        return ERROR_NOT_SUPPORTED;
-    }
-    case DISPLAYCONFIG_DEVICE_INFO_GET_ADAPTER_NAME:
-    {
-        DISPLAYCONFIG_ADAPTER_NAME *adapter_name = (DISPLAYCONFIG_ADAPTER_NAME *)packet;
-
-        FIXME("DISPLAYCONFIG_DEVICE_INFO_GET_ADAPTER_NAME: stub\n");
-
-        if (packet->size < sizeof(*adapter_name))
-            return ERROR_INVALID_PARAMETER;
-
-        return ERROR_NOT_SUPPORTED;
-    }
-    case DISPLAYCONFIG_DEVICE_INFO_SET_TARGET_PERSISTENCE:
-    case DISPLAYCONFIG_DEVICE_INFO_GET_TARGET_BASE_TYPE:
-    case DISPLAYCONFIG_DEVICE_INFO_GET_SUPPORT_VIRTUAL_RESOLUTION:
-    case DISPLAYCONFIG_DEVICE_INFO_SET_SUPPORT_VIRTUAL_RESOLUTION:
-    case DISPLAYCONFIG_DEVICE_INFO_GET_ADVANCED_COLOR_INFO:
-    case DISPLAYCONFIG_DEVICE_INFO_SET_ADVANCED_COLOR_STATE:
-    case DISPLAYCONFIG_DEVICE_INFO_GET_SDR_WHITE_LEVEL:
-    default:
-        FIXME("Unimplemented packet type: %u\n", packet->type);
-        return ERROR_INVALID_PARAMETER;
-    }
+    return RtlNtStatusToDosError(NtUserDisplayConfigGetDeviceInfo(packet));
 }
 
 /***********************************************************************
