@@ -913,7 +913,6 @@ DWORD get_input_state(void)
 static HKL get_locale_kbd_layout(void)
 {
     LCID layout;
-    LANGID langid;
 
     /* FIXME:
      *
@@ -927,19 +926,7 @@ static HKL get_locale_kbd_layout(void)
      */
 
     NtQueryDefaultLocale( TRUE, &layout );
-
-    /*
-     * Microsoft Office expects this value to be something specific
-     * for Japanese and Korean Windows with an IME the value is 0xe001
-     * We should probably check to see if an IME exists and if so then
-     * set this word properly.
-     */
-    langid = PRIMARYLANGID( LANGIDFROMLCID( layout ) );
-    if (langid == LANG_CHINESE || langid == LANG_JAPANESE || langid == LANG_KOREAN)
-        layout = MAKELONG( layout, 0xe001 ); /* IME */
-    else
-        layout = MAKELONG( layout, layout );
-
+    layout = MAKELONG( layout, layout );
     return ULongToHandle( layout );
 }
 
@@ -1187,27 +1174,25 @@ INT WINAPI NtUserToUnicodeEx( UINT virt, UINT scan, const BYTE *state,
                               WCHAR *str, int size, UINT flags, HKL layout )
 {
     const KBDTABLES *kbd_tables;
-    WCHAR buffer[2] = {0};
     INT len;
 
     TRACE_(keyboard)( "virt %#x, scan %#x, state %p, str %p, size %d, flags %#x, layout %p.\n",
                       virt, scan, state, str, size, flags, layout );
 
-    if (!state) return 0;
+    if (!state || !size) return 0;
     if ((len = user_driver->pToUnicodeEx( virt, scan, state, str, size, flags, layout )) >= -1) return len;
 
     if (!(kbd_tables = user_driver->pKbdLayerDescriptor( layout ))) kbd_tables = &kbdus_tables;
-    if (scan & 0x8000) buffer[0] = 0; /* key up */
-    else buffer[0] = kbd_tables_vkey_to_wchar( kbd_tables, virt, state );
+    if (scan & 0x8000) str[0] = 0; /* key up */
+    else str[0] = kbd_tables_vkey_to_wchar( kbd_tables, virt, state );
+    if (size > 1) str[1] = 0;
 
-    if (buffer[0] != WCH_NONE) len = 1;
-    else buffer[0] = len = 0;
-
-    lstrcpynW( str, buffer, size );
+    if (str[0] != WCH_NONE) len = 1;
+    else str[0] = len = 0;
 
     if (kbd_tables != &kbdus_tables) user_driver->pReleaseKbdTables( kbd_tables );
 
-    TRACE_(keyboard)( "ret %d, str %s.\n", len, debugstr_w(str) );
+    TRACE_(keyboard)( "ret %d, str %s.\n", len, debugstr_wn(str, len) );
     return len;
 }
 
@@ -1795,13 +1780,10 @@ HWND WINAPI NtUserSetCapture( HWND hwnd )
  */
 BOOL release_capture(void)
 {
-    HWND previous = NULL;
-    BOOL ret;
-
-    ret = set_capture_window( 0, 0, &previous );
+    BOOL ret = set_capture_window( 0, 0, NULL );
 
     /* Somebody may have missed some mouse movements */
-    if (ret && previous)
+    if (ret)
     {
         INPUT input = { .type = INPUT_MOUSE };
         input.mi.dwFlags = MOUSEEVENTF_MOVE;
