@@ -170,8 +170,7 @@ static MACDRV_PDEVICE *create_mac_physdev(void)
 /**********************************************************************
  *              CreateDC (MACDRV.@)
  */
-static BOOL CDECL macdrv_CreateDC(PHYSDEV *pdev, LPCWSTR device, LPCWSTR output,
-                                  const DEVMODEW* initData)
+static BOOL macdrv_CreateDC(PHYSDEV *pdev, LPCWSTR device, LPCWSTR output, const DEVMODEW* initData)
 {
     MACDRV_PDEVICE *physDev = create_mac_physdev();
 
@@ -188,7 +187,7 @@ static BOOL CDECL macdrv_CreateDC(PHYSDEV *pdev, LPCWSTR device, LPCWSTR output,
 /**********************************************************************
  *              CreateCompatibleDC (MACDRV.@)
  */
-static BOOL CDECL macdrv_CreateCompatibleDC(PHYSDEV orig, PHYSDEV *pdev)
+static BOOL macdrv_CreateCompatibleDC(PHYSDEV orig, PHYSDEV *pdev)
 {
     MACDRV_PDEVICE *physDev = create_mac_physdev();
 
@@ -205,7 +204,7 @@ static BOOL CDECL macdrv_CreateCompatibleDC(PHYSDEV orig, PHYSDEV *pdev)
 /**********************************************************************
  *              DeleteDC (MACDRV.@)
  */
-static BOOL CDECL macdrv_DeleteDC(PHYSDEV dev)
+static BOOL macdrv_DeleteDC(PHYSDEV dev)
 {
     MACDRV_PDEVICE *physDev = get_macdrv_dev(dev);
 
@@ -219,7 +218,7 @@ static BOOL CDECL macdrv_DeleteDC(PHYSDEV dev)
 /***********************************************************************
  *              GetDeviceCaps (MACDRV.@)
  */
-static INT CDECL macdrv_GetDeviceCaps(PHYSDEV dev, INT cap)
+static INT macdrv_GetDeviceCaps(PHYSDEV dev, INT cap)
 {
     INT ret;
 
@@ -270,8 +269,9 @@ static const struct user_driver_funcs macdrv_funcs =
     .pBeep = macdrv_Beep,
     .pChangeDisplaySettings = macdrv_ChangeDisplaySettings,
     .pClipCursor = macdrv_ClipCursor,
+    .pNotifyIcon = macdrv_NotifyIcon,
+    .pCleanupIcons = macdrv_CleanupIcons,
     .pClipboardWindowProc = macdrv_ClipboardWindowProc,
-    .pCreateDesktopWindow = macdrv_CreateDesktopWindow,
     .pDesktopWindowProc = macdrv_DesktopWindowProc,
     .pDestroyCursorIcon = macdrv_DestroyCursorIcon,
     .pDestroyWindow = macdrv_DestroyWindow,
@@ -282,11 +282,12 @@ static const struct user_driver_funcs macdrv_funcs =
     .pGetKeyboardLayoutList = macdrv_GetKeyboardLayoutList,
     .pGetKeyNameText = macdrv_GetKeyNameText,
     .pMapVirtualKeyEx = macdrv_MapVirtualKeyEx,
-    .pMsgWaitForMultipleObjectsEx = macdrv_MsgWaitForMultipleObjectsEx,
+    .pProcessEvents = macdrv_ProcessEvents,
     .pRegisterHotKey = macdrv_RegisterHotKey,
     .pSetCapture = macdrv_SetCapture,
     .pSetCursor = macdrv_SetCursor,
     .pSetCursorPos = macdrv_SetCursorPos,
+    .pSetDesktopWindow = macdrv_SetDesktopWindow,
     .pSetFocus = macdrv_SetFocus,
     .pSetLayeredWindowAttributes = macdrv_SetLayeredWindowAttributes,
     .pSetParent = macdrv_SetParent,
@@ -302,6 +303,8 @@ static const struct user_driver_funcs macdrv_funcs =
     .pUpdateClipboard = macdrv_UpdateClipboard,
     .pUpdateLayeredWindow = macdrv_UpdateLayeredWindow,
     .pVkKeyScanEx = macdrv_VkKeyScanEx,
+    .pImeProcessKey = macdrv_ImeProcessKey,
+    .pNotifyIMEStatus = macdrv_NotifyIMEStatus,
     .pWindowMessage = macdrv_WindowMessage,
     .pWindowPosChanged = macdrv_WindowPosChanged,
     .pWindowPosChanging = macdrv_WindowPosChanging,
