@@ -41,6 +41,10 @@ typedef enum _DISPLAYORDER
     Date
 } DISPLAYORDER;
 
+#define MAX_DATETIME_FORMAT 80
+
+static WCHAR date_format[MAX_DATETIME_FORMAT * 2];
+static WCHAR time_format[MAX_DATETIME_FORMAT * 2];
 static int file_total, dir_total, max_width;
 static ULONGLONG byte_total;
 static DISPLAYTIME dirTime;
@@ -49,25 +53,6 @@ static BOOL orderReverse, orderGroupDirs, orderGroupDirsReverse, orderByCol;
 static BOOL paged_mode, recurse, wide, bare, lower, shortname, usernames, separator;
 static ULONG showattrs, attrsbits;
 
-/*****************************************************************************
- * WCMD_strrev
- *
- * Reverse a WCHARacter string in-place (strrev() is not available under unixen :-( ).
- */
-static WCHAR * WCMD_strrev (WCHAR *buff) {
-
-  int r, i;
-  WCHAR b;
-
-  r = lstrlenW (buff);
-  for (i=0; i<r/2; i++) {
-    b = buff[i];
-    buff[i] = buff[r-i-1];
-    buff[r-i-1] = b;
-  }
-  return (buff);
-}
-
 /*****************************************************************************
  * WCMD_filesize64
  *
@@ -92,7 +77,7 @@ static WCHAR * WCMD_filesize64 (ULONGLONG n) {
     *p = '\0';
     n = q;
   } while (n != 0);
-  WCMD_strrev (buff);
+  wcsrev(buff);
   return buff;
 }
 
@@ -166,8 +151,8 @@ static int __cdecl WCMD_dir_sort (const void *a, const void *b)
       WCHAR extB[MAX_PATH];
 
       /* Split into components */
-      WCMD_splitpath(filea->cFileName, drive, dir, fname, extA);
-      WCMD_splitpath(fileb->cFileName, drive, dir, fname, extB);
+      _wsplitpath(filea->cFileName, drive, dir, fname, extA);
+      _wsplitpath(fileb->cFileName, drive, dir, fname, extB);
       result = lstrcmpiW(extA, extB);
   }
 
@@ -177,8 +162,6 @@ static int __cdecl WCMD_dir_sort (const void *a, const void *b)
 
 /*****************************************************************************
  * WCMD_getfileowner
- *
- * Reverse a WCHARacter string in-place (strrev() is not available under unixen :-( ).
  */
 static void WCMD_getfileowner(WCHAR *filename, WCHAR *owner, int ownerlen) {
 
@@ -203,27 +186,26 @@ static void WCMD_getfileowner(WCHAR *filename, WCHAR *owner, int ownerlen) {
         ULONG domainLen = MAXSTRING;
         SID_NAME_USE nameuse;
 
-        secBuffer = heap_xalloc(sizeNeeded * sizeof(BYTE));
+        secBuffer = xalloc(sizeNeeded * sizeof(BYTE));
 
         /* Get the owners security descriptor */
         if(!GetFileSecurityW(filename, OWNER_SECURITY_INFORMATION, secBuffer,
                             sizeNeeded, &sizeNeeded)) {
-            heap_free(secBuffer);
+            free(secBuffer);
             return;
         }
 
         /* Get the SID from the SD */
         if(!GetSecurityDescriptorOwner(secBuffer, &pSID, &defaulted)) {
-            heap_free(secBuffer);
+            free(secBuffer);
             return;
         }
 
         /* Convert to a username */
         if (LookupAccountSidW(NULL, pSID, name, &nameLen, domain, &domainLen, &nameuse)) {
-            static const WCHAR fmt[]  = {'%','s','%','c','%','s','\0'};
-            swprintf(owner, ownerlen, fmt, domain, '\\', name);
+            swprintf(owner, ownerlen, L"%s%c%s", domain, '\\', name);
         }
-        heap_free(secBuffer);
+        free(secBuffer);
     }
     return;
 }
@@ -253,15 +235,6 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
   int concurrentDirs = 0;
   BOOL done_header = FALSE;
 
-  static const WCHAR fmtDir[]  = {'%','1','!','1','0','s','!',' ',' ','%','2','!','8','s','!',' ',' ',
-                                  '<','D','I','R','>',' ',' ',' ',' ',' ',' ',' ',' ',' ','\0'};
-  static const WCHAR fmtFile[] = {'%','1','!','1','0','s','!',' ',' ','%','2','!','8','s','!',' ',' ',
-                                  ' ',' ','%','3','!','1','0','s','!',' ',' ','\0'};
-  static const WCHAR fmt2[]  = {'%','1','!','-','1','3','s','!','\0'};
-  static const WCHAR fmt3[]  = {'%','1','!','-','2','3','s','!','\0'};
-  static const WCHAR fmt4[]  = {'%','1','\0'};
-  static const WCHAR fmt5[]  = {'%','1','%','2','\0'};
-
   dir_count = 0;
   file_count = 0;
   entry_count = 0;
@@ -273,7 +246,7 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
      same directory. Note issuing a directory header with no contents
      mirrors what windows does                                            */
   parms = inputparms;
-  fd = heap_xalloc(sizeof(WIN32_FIND_DATAW));
+  fd = xalloc(sizeof(WIN32_FIND_DATAW));
   while (parms && lstrcmpW(inputparms->dirName, parms->dirName) == 0) {
     concurrentDirs++;
 
@@ -298,7 +271,7 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
            if (tmpLen > widest) widest = tmpLen;
         }
 
-        fd = HeapReAlloc(GetProcessHeap(),0,fd,(entry_count+1)*sizeof(WIN32_FIND_DATAW));
+        fd = realloc(fd, (entry_count + 1) * sizeof(WIN32_FIND_DATAW));
         if (fd == NULL) {
           FindClose (hff);
           WINE_ERR("Out of memory\n");
@@ -315,11 +288,9 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
 
     /* Output the results */
     if (!bare) {
-       if (level != 0 && (entry_count > 0)) WCMD_output_asis (newlineW);
+       if (level != 0 && (entry_count > 0)) WCMD_output_asis(L"\r\n");
        if (!recurse || ((entry_count > 0) && done_header==FALSE)) {
-           static const WCHAR headerW[] = {'D','i','r','e','c','t','o','r','y',' ','o','f',
-                                           ' ','%','1','\n','\n','\0'};
-           WCMD_output (headerW, real_path);
+           WCMD_output (L"Directory of %1\n\n", real_path);
            done_header = TRUE;
        }
     }
@@ -361,10 +332,7 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
       }
 
       /* /L convers all names to lower case */
-      if (lower) {
-          WCHAR *p = fd[i].cFileName;
-          while ( (*p = tolower(*p)) ) ++p;
-      }
+      if (lower) wcslwr( fd[i].cFileName );
 
       /* /Q gets file ownership information */
       if (usernames) {
@@ -381,20 +349,20 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
         FileTimeToLocalFileTime (&fd[i].ftCreationTime, &ft);
       }
       FileTimeToSystemTime (&ft, &st);
-      GetDateFormatW(0, DATE_SHORTDATE, &st, NULL, datestring, ARRAY_SIZE(datestring));
-      GetTimeFormatW(0, TIME_NOSECONDS, &st, NULL, timestring, ARRAY_SIZE(timestring));
+      GetDateFormatW(LOCALE_USER_DEFAULT, 0, &st, date_format,
+                     datestring, ARRAY_SIZE(datestring));
+      GetTimeFormatW(LOCALE_USER_DEFAULT, TIME_NOSECONDS, &st, time_format,
+                     timestring, ARRAY_SIZE(timestring));
 
       if (wide) {
 
         tmp_width = cur_width;
         if (fd[i].dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
-            static const WCHAR fmt[] = {'[','%','1',']','\0'};
-            WCMD_output (fmt, fd[i].cFileName);
+            WCMD_output (L"[%1]", fd[i].cFileName);
             dir_count++;
             tmp_width = tmp_width + lstrlenW(fd[i].cFileName) + 2;
         } else {
-            static const WCHAR fmt[] = {'%','1','\0'};
-            WCMD_output (fmt, fd[i].cFileName);
+            WCMD_output (L"%1", fd[i].cFileName);
             tmp_width = tmp_width + lstrlenW(fd[i].cFileName) ;
             file_count++;
             file_size.u.LowPart = fd[i].nFileSizeLow;
@@ -406,22 +374,21 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
         if ((cur_width + widest) > max_width) {
             cur_width = 0;
         } else {
-            static const WCHAR padfmt[] = {'%','1','!','*','s','!','\0'};
-            WCMD_output(padfmt, cur_width - tmp_width, nullW);
+            WCMD_output(L"%1!*s!", cur_width - tmp_width, L"");
         }
 
       } else if (fd[i].dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
         dir_count++;
 
         if (!bare) {
-           WCMD_output (fmtDir, datestring, timestring);
-           if (shortname) WCMD_output (fmt2, fd[i].cAlternateFileName);
-           if (usernames) WCMD_output (fmt3, username);
-           WCMD_output(fmt4,fd[i].cFileName);
+           WCMD_output (L"%1  %2    <DIR>          ", datestring, timestring);
+           if (shortname) WCMD_output(L"%1!-13s!", fd[i].cAlternateFileName);
+           if (usernames) WCMD_output(L"%1!-23s!", username);
+           WCMD_output(L"%1",fd[i].cFileName);
         } else {
-           if (!((lstrcmpW(fd[i].cFileName, dotW) == 0) ||
-                 (lstrcmpW(fd[i].cFileName, dotdotW) == 0))) {
-              WCMD_output (fmt5, recurse?inputparms->dirName:nullW, fd[i].cFileName);
+           if (!((lstrcmpW(fd[i].cFileName, L".") == 0) ||
+                 (lstrcmpW(fd[i].cFileName, L"..") == 0))) {
+              WCMD_output(L"%1%2", recurse ? inputparms->dirName : L"", fd[i].cFileName);
            } else {
               addNewLine = FALSE;
            }
@@ -433,30 +400,26 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
         file_size.u.HighPart = fd[i].nFileSizeHigh;
         byte_count.QuadPart += file_size.QuadPart;
         if (!bare) {
-           WCMD_output (fmtFile, datestring, timestring,
+           WCMD_output (L"%1  %2    %3!14s! ", datestring, timestring,
                         WCMD_filesize64(file_size.QuadPart));
-           if (shortname) WCMD_output (fmt2, fd[i].cAlternateFileName);
-           if (usernames) WCMD_output (fmt3, username);
-           WCMD_output(fmt4,fd[i].cFileName);
+           if (shortname) WCMD_output(L"%1!-13s!", fd[i].cAlternateFileName);
+           if (usernames) WCMD_output(L"%1!-23s!", username);
+           WCMD_output(L"%1",fd[i].cFileName);
         } else {
-           WCMD_output (fmt5, recurse?inputparms->dirName:nullW, fd[i].cFileName);
+           WCMD_output(L"%1%2", recurse ? inputparms->dirName : L"", fd[i].cFileName);
         }
       }
      }
-     if (addNewLine) WCMD_output_asis (newlineW);
+     if (addNewLine) WCMD_output_asis(L"\r\n");
      cur_width = 0;
     }
 
     if (!bare) {
        if (file_count == 1) {
-         static const WCHAR fmt[] = {' ',' ',' ',' ',' ',' ',' ','1',' ','f','i','l','e',' ',
-                                     '%','1','!','2','5','s','!',' ','b','y','t','e','s','\n','\0'};
-         WCMD_output (fmt, WCMD_filesize64 (byte_count.QuadPart));
+         WCMD_output (L"       1 file %1!25s! bytes\n", WCMD_filesize64 (byte_count.QuadPart));
        }
        else {
-         static const WCHAR fmt[] = {'%','1','!','8','d','!',' ','f','i','l','e','s',' ','%','2','!','2','4','s','!',
-                                     ' ','b','y','t','e','s','\n','\0'};
-         WCMD_output (fmt, file_count, WCMD_filesize64 (byte_count.QuadPart));
+         WCMD_output (L"%1!8d! files %2!24s! bytes\n", file_count, WCMD_filesize64 (byte_count.QuadPart));
        }
     }
     byte_total = byte_total + byte_count.QuadPart;
@@ -465,17 +428,13 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
 
     if (!bare && !recurse) {
        if (dir_count == 1) {
-           static const WCHAR fmt[] = {'%','1','!','8','d','!',' ','d','i','r','e','c','t','o','r','y',
-                                       ' ',' ',' ',' ',' ',' ',' ',' ',' ','\0'};
-           WCMD_output (fmt, 1);
+           WCMD_output (L"%1!8d! directory         ", 1);
        } else {
-           static const WCHAR fmt[] = {'%','1','!','8','d','!',' ','d','i','r','e','c','t','o','r','i',
-                                       'e','s','\0'};
-           WCMD_output (fmt, dir_count);
+           WCMD_output (L"%1!8d! directories", dir_count);
        }
     }
   }
-  heap_free(fd);
+  free(fd);
 
   /* When recursing, look in all subdirectories for matches */
   if (recurse) {
@@ -485,15 +444,15 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
 
     /* Build path to search */
     lstrcpyW(string, inputparms->dirName);
-    lstrcatW(string, starW);
+    lstrcatW(string, L"*");
 
     WINE_TRACE("Recursive, looking for '%s'\n", wine_dbgstr_w(string));
     hff = FindFirstFileW(string, &finddata);
     if (hff != INVALID_HANDLE_VALUE) {
       do {
         if ((finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
-            (lstrcmpW(finddata.cFileName, dotdotW) != 0) &&
-            (lstrcmpW(finddata.cFileName, dotW) != 0)) {
+            (lstrcmpW(finddata.cFileName, L"..") != 0) &&
+            (lstrcmpW(finddata.cFileName, L".") != 0)) {
 
           DIRECTORY_STACK *thisDir;
           int              dirsToCopy = concurrentDirs;
@@ -506,17 +465,17 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
             /* Work out search parameter in sub dir */
             lstrcpyW (string, inputparms->dirName);
             lstrcatW (string, finddata.cFileName);
-            lstrcatW (string, slashW);
+            lstrcatW(string, L"\\");
             WINE_TRACE("Recursive, Adding to search list '%s'\n", wine_dbgstr_w(string));
 
             /* Allocate memory, add to list */
-            thisDir = heap_xalloc(sizeof(DIRECTORY_STACK));
+            thisDir = xalloc(sizeof(DIRECTORY_STACK));
             if (dirStack == NULL) dirStack = thisDir;
             if (lastEntry != NULL) lastEntry->next = thisDir;
             lastEntry = thisDir;
             thisDir->next = NULL;
-            thisDir->dirName = heap_strdupW(string);
-            thisDir->fileName = heap_strdupW(parms->fileName);
+            thisDir->dirName = xstrdupW(string);
+            thisDir->fileName = xstrdupW(parms->fileName);
             parms = parms->next;
           }
         }
@@ -528,9 +487,9 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
         dirStack = WCMD_list_directory (thisDir, 1);
         while (thisDir != dirStack) {
           DIRECTORY_STACK *tempDir = thisDir->next;
-          heap_free(thisDir->dirName);
-          heap_free(thisDir->fileName);
-          heap_free(thisDir);
+          free(thisDir->dirName);
+          free(thisDir->fileName);
+          free(thisDir);
           thisDir = tempDir;
         }
       }
@@ -550,36 +509,127 @@ static DIRECTORY_STACK *WCMD_list_directory (DIRECTORY_STACK *inputparms, int le
 /*****************************************************************************
  * WCMD_dir_trailer
  *
- * Print out the trailer for the supplied drive letter
+ * Print out the trailer for the supplied path
  */
-static void WCMD_dir_trailer(WCHAR drive) {
-    ULARGE_INTEGER avail, total, freebytes;
-    DWORD status;
-    WCHAR driveName[] = {'c',':','\\','\0'};
+static void WCMD_dir_trailer(const WCHAR *path) {
+    ULARGE_INTEGER freebytes;
+    BOOL status;
 
-    driveName[0] = drive;
-    status = GetDiskFreeSpaceExW(driveName, &avail, &total, &freebytes);
-    WINE_TRACE("Writing trailer for '%s' gave %d(%d)\n", wine_dbgstr_w(driveName),
+    status = GetDiskFreeSpaceExW(path, NULL, NULL, &freebytes);
+    WINE_TRACE("Writing trailer for '%s' gave %d(%ld)\n", wine_dbgstr_w(path),
                status, GetLastError());
 
     if (errorlevel==0 && !bare) {
       if (recurse) {
-        static const WCHAR fmt1[] = {'\n',' ',' ',' ',' ',' ','T','o','t','a','l',' ','f','i','l','e','s',
-                                     ' ','l','i','s','t','e','d',':','\n','%','1','!','8','d','!',' ','f','i','l','e',
-                                     's','%','2','!','2','5','s','!',' ','b','y','t','e','s','\n','\0'};
-        static const WCHAR fmt2[] = {'%','1','!','8','d','!',' ','d','i','r','e','c','t','o','r','i','e','s',' ','%',
-                                     '2','!','1','8','s','!',' ','b','y','t','e','s',' ','f','r','e','e','\n','\n',
-                                     '\0'};
-        WCMD_output (fmt1, file_total, WCMD_filesize64 (byte_total));
-        WCMD_output (fmt2, dir_total, WCMD_filesize64 (freebytes.QuadPart));
+        WCMD_output (L"\n     Total files listed:\n%1!8d! files%2!25s! bytes\n", file_total, WCMD_filesize64 (byte_total));
+        WCMD_output (L"%1!8d! directories %2!18s! bytes free\n\n", dir_total, WCMD_filesize64 (freebytes.QuadPart));
       } else {
-        static const WCHAR fmt[] = {' ','%','1','!','1','8','s','!',' ','b','y','t','e','s',' ','f','r','e','e',
-                                    '\n','\n','\0'};
-        WCMD_output (fmt, WCMD_filesize64 (freebytes.QuadPart));
+        WCMD_output (L" %1!18s! bytes free\n\n", WCMD_filesize64 (freebytes.QuadPart));
       }
     }
 }
 
+/* Get the length of a date/time formatting pattern */
+/* copied from dlls/kernelbase/locale.c */
+static int get_pattern_len( const WCHAR *pattern, const WCHAR *accept )
+{
+    int i;
+
+    if (*pattern == '\'')
+    {
+        for (i = 1; pattern[i]; i++)
+        {
+            if (pattern[i] != '\'') continue;
+            if (pattern[++i] != '\'') return i;
+        }
+        return i;
+    }
+    if (!wcschr( accept, *pattern )) return 1;
+    for (i = 1; pattern[i]; i++) if (pattern[i] != pattern[0]) break;
+    return i;
+}
+
+/* Initialize date format to use abbreviated one with leading zeros */
+static void init_date_format(void)
+{
+    WCHAR sys_format[MAX_DATETIME_FORMAT];
+    int src_pat_len, dst_pat_len;
+    const WCHAR *src;
+    WCHAR *dst = date_format;
+
+    GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_SSHORTDATE, sys_format, ARRAY_SIZE(sys_format));
+
+    for (src = sys_format; *src; src += src_pat_len, dst += dst_pat_len) {
+        src_pat_len = dst_pat_len = get_pattern_len(src, L"yMd");
+
+        switch (*src)
+        {
+        case '\'':
+            wmemcpy(dst, src, src_pat_len);
+            break;
+
+        case 'd':
+        case 'M':
+            if (src_pat_len == 4) /* full name */
+                dst_pat_len--; /* -> use abbreviated one */
+            /* fallthrough */
+        case 'y':
+            if (src_pat_len == 1) /* without leading zeros */
+                dst_pat_len++; /* -> with leading zeros */
+            wmemset(dst, *src, dst_pat_len);
+            break;
+
+        default:
+            *dst = *src;
+            break;
+        }
+    }
+    *dst = '\0';
+
+    TRACE("date format: %s\n", wine_dbgstr_w(date_format));
+}
+
+/* Initialize time format to use leading zeros */
+static void init_time_format(void)
+{
+    WCHAR sys_format[MAX_DATETIME_FORMAT];
+    int src_pat_len, dst_pat_len;
+    const WCHAR *src;
+    WCHAR *dst = time_format;
+
+    GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_STIMEFORMAT, sys_format, ARRAY_SIZE(sys_format));
+
+    for (src = sys_format; *src; src += src_pat_len, dst += dst_pat_len) {
+        src_pat_len = dst_pat_len = get_pattern_len(src, L"Hhmst");
+
+        switch (*src)
+        {
+        case '\'':
+            wmemcpy(dst, src, src_pat_len);
+            break;
+
+        case 'H':
+        case 'h':
+        case 'm':
+        case 's':
+            if (src_pat_len == 1) /* without leading zeros */
+                dst_pat_len++; /* -> with leading zeros */
+            /* fallthrough */
+        case 't':
+            wmemset(dst, *src, dst_pat_len);
+            break;
+
+        default:
+            *dst = *src;
+            break;
+        }
+    }
+    *dst = '\0';
+
+    /* seconds portion will be dropped by TIME_NOSECONDS */
+    TRACE("time format: %s\n", wine_dbgstr_w(time_format));
+}
+
 /*****************************************************************************
  * WCMD_directory
  *
@@ -605,14 +655,12 @@ void WCMD_directory (WCHAR *args)
   WCHAR dir[MAX_PATH];
   WCHAR fname[MAX_PATH];
   WCHAR ext[MAX_PATH];
-  static const WCHAR dircmdW[] = {'D','I','R','C','M','D','\0'};
 
   errorlevel = 0;
 
   /* Prefill quals with (uppercased) DIRCMD env var */
-  if (GetEnvironmentVariableW(dircmdW, string, ARRAY_SIZE(string))) {
-    p = string;
-    while ( (*p = toupper(*p)) ) ++p;
+  if (GetEnvironmentVariableW(L"DIRCMD", string, ARRAY_SIZE(string))) {
+    wcsupr( string );
     lstrcatW(string,quals);
     lstrcpyW(quals, string);
   }
@@ -759,7 +807,7 @@ void WCMD_directory (WCHAR *args)
                 p++;
               }
               p = p - 1; /* So when step on, move to '/' */
-              WINE_TRACE("Result: showattrs %x, bits %x\n", showattrs, attrsbits);
+              WINE_TRACE("Result: showattrs %lx, bits %lx\n", showattrs, attrsbits);
               break;
     default:
               SetLastError(ERROR_INVALID_PARAMETER);
@@ -786,10 +834,13 @@ void WCMD_directory (WCHAR *args)
      WCMD_enter_paged_mode(NULL);
   }
 
+  init_date_format();
+  init_time_format();
+
   argno         = 0;
   argN          = args;
   GetCurrentDirectoryW(MAX_PATH, cwd);
-  lstrcatW(cwd, slashW);
+  lstrcatW(cwd, L"\\");
 
   /* Loop through all args, calculating full effective directory */
   fullParms = NULL;
@@ -804,13 +855,11 @@ void WCMD_directory (WCHAR *args)
         lstrcpyW(fullname, thisArg);
       } else if (thisArg[1] == ':' && thisArg[2] != '\\') {
         WCHAR envvar[4];
-        static const WCHAR envFmt[] = {'=','%','c',':','\0'};
-        wsprintfW(envvar, envFmt, thisArg[0]);
+        wsprintfW(envvar, L"=%c:", thisArg[0]);
         if (!GetEnvironmentVariableW(envvar, fullname, MAX_PATH)) {
-          static const WCHAR noEnvFmt[] = {'%','c',':','\0'};
-          wsprintfW(fullname, noEnvFmt, thisArg[0]);
+          wsprintfW(fullname, L"%c:", thisArg[0]);
         }
-        lstrcatW(fullname, slashW);
+        lstrcatW(fullname, L"\\");
         lstrcatW(fullname, &thisArg[2]);
       } else if (thisArg[0] == '\\') {
         memcpy(fullname, cwd, 2 * sizeof(WCHAR));
@@ -821,7 +870,7 @@ void WCMD_directory (WCHAR *args)
       }
       WINE_TRACE("Using location '%s'\n", wine_dbgstr_w(fullname));
 
-      status = GetFullPathNameW(fullname, ARRAY_SIZE(path), path, NULL);
+      if (!WCMD_get_fullpath(fullname, ARRAY_SIZE(path), path, NULL)) continue;
 
       /*
        *  If the path supplied does not include a wildcard, and the endpoint of the
@@ -831,33 +880,33 @@ void WCMD_directory (WCHAR *args)
       if ((wcschr(path, '*') == NULL) && (wcschr(path, '%') == NULL)) {
         status = GetFileAttributesW(path);
         if ((status != INVALID_FILE_ATTRIBUTES) && (status & FILE_ATTRIBUTE_DIRECTORY)) {
-          if (!ends_with_backslash( path )) lstrcatW( path, slashW );
-          lstrcatW (path, starW);
+          if (!ends_with_backslash(path)) lstrcatW(path, L"\\");
+          lstrcatW(path, L"*");
         }
       } else {
         /* Special case wildcard search with no extension (ie parameters ending in '.') as
            GetFullPathName strips off the additional '.'                                  */
-        if (fullname[lstrlenW(fullname)-1] == '.') lstrcatW(path, dotW);
+        if (fullname[lstrlenW(fullname)-1] == '.') lstrcatW(path, L".");
       }
 
       WINE_TRACE("Using path '%s'\n", wine_dbgstr_w(path));
-      thisEntry = heap_xalloc(sizeof(DIRECTORY_STACK));
+      thisEntry = xalloc(sizeof(DIRECTORY_STACK));
       if (fullParms == NULL) fullParms = thisEntry;
       if (prevEntry != NULL) prevEntry->next = thisEntry;
       prevEntry = thisEntry;
       thisEntry->next = NULL;
 
       /* Split into components */
-      WCMD_splitpath(path, drive, dir, fname, ext);
+      _wsplitpath(path, drive, dir, fname, ext);
       WINE_TRACE("Path Parts: drive: '%s' dir: '%s' name: '%s' ext:'%s'\n",
                  wine_dbgstr_w(drive), wine_dbgstr_w(dir),
                  wine_dbgstr_w(fname), wine_dbgstr_w(ext));
 
-      thisEntry->dirName = heap_xalloc(sizeof(WCHAR) * (lstrlenW(drive)+lstrlenW(dir)+1));
+      thisEntry->dirName = xalloc(sizeof(WCHAR) * (wcslen(drive) + wcslen(dir) + 1));
       lstrcpyW(thisEntry->dirName, drive);
       lstrcatW(thisEntry->dirName, dir);
 
-      thisEntry->fileName = heap_xalloc(sizeof(WCHAR) * (lstrlenW(fname)+lstrlenW(ext)+1));
+      thisEntry->fileName = xalloc(sizeof(WCHAR) * (wcslen(fname) + wcslen(ext) + 1));
       lstrcpyW(thisEntry->fileName, fname);
       lstrcatW(thisEntry->fileName, ext);
 
@@ -867,10 +916,10 @@ void WCMD_directory (WCHAR *args)
   /* If just 'dir' entered, a '*' parameter is assumed */
   if (fullParms == NULL) {
     WINE_TRACE("Inserting default '*'\n");
-    fullParms = heap_xalloc(sizeof(DIRECTORY_STACK));
+    fullParms = xalloc(sizeof(DIRECTORY_STACK));
     fullParms->next = NULL;
-    fullParms->dirName = heap_strdupW(cwd);
-    fullParms->fileName = heap_strdupW(starW);
+    fullParms->dirName = xstrdupW(cwd);
+    fullParms->fileName = xstrdupW(L"*");
   }
 
   lastDrive = '?';
@@ -882,15 +931,16 @@ void WCMD_directory (WCHAR *args)
 
     /* Output disk free (trailer) and volume information (header) if the drive
        letter changes */
-    if (lastDrive != toupper(thisEntry->dirName[0])) {
+    if (lastDrive != towupper(thisEntry->dirName[0])) {
 
       /* Trailer Information */
       if (lastDrive != '?') {
         trailerReqd = FALSE;
-        WCMD_dir_trailer(prevEntry->dirName[0]);
+        WCMD_dir_trailer(prevEntry->dirName);
+        byte_total = file_total = dir_total = 0;
       }
 
-      lastDrive = toupper(thisEntry->dirName[0]);
+      lastDrive = towupper(thisEntry->dirName[0]);
 
       if (!bare) {
          WCHAR drive[3];
@@ -906,8 +956,7 @@ void WCMD_directory (WCHAR *args)
          }
       }
     } else {
-      static const WCHAR newLine2[] = {'\n','\n','\0'};
-      if (!bare) WCMD_output_asis (newLine2);
+      if (!bare) WCMD_output_asis (L"\n\n");
     }
 
     /* Clear any errors from previous invocations, and process it */
@@ -918,7 +967,7 @@ void WCMD_directory (WCHAR *args)
 
   /* Trailer Information */
   if (trailerReqd) {
-    WCMD_dir_trailer(prevEntry->dirName[0]);
+    WCMD_dir_trailer(prevEntry->dirName);
   }
 
 exit:
@@ -928,8 +977,8 @@ exit:
   while (fullParms != NULL) {
     prevEntry = fullParms;
     fullParms = prevEntry->next;
-    heap_free(prevEntry->dirName);
-    heap_free(prevEntry->fileName);
-    heap_free(prevEntry);
+    free(prevEntry->dirName);
+    free(prevEntry->fileName);
+    free(prevEntry);
   }
 }
